@@ -2,9 +2,6 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-// This file is loaded into the browser window scope.
-/* eslint-env mozilla/browser-window */
-
 var gProfiles = {
   async init() {
     this.createNewProfile = this.createNewProfile.bind(this);
@@ -106,7 +103,7 @@ var gProfiles = {
     );
     let avatarURL =
       await SelectableProfileService.currentProfile.getAvatarURL(16);
-    profilesButton.setAttribute("image", `${avatarURL}`);
+    profilesButton.setAttribute("image", avatarURL);
   },
 
   /**
@@ -114,22 +111,25 @@ var gProfiles = {
    */
   async onPopupShowing() {
     let menuPopup = document.getElementById("menu_ProfilesPopup");
-    while (menuPopup.hasChildNodes()) {
-      menuPopup.firstChild.remove();
-    }
-
     let profiles = await SelectableProfileService.getAllProfiles();
     let currentProfile = SelectableProfileService.currentProfile;
-
+    let insertionPoint = document.getElementById("menu_newProfile");
+    let existingItems = [
+      ...menuPopup.querySelectorAll(":scope > menuitem[profileid]"),
+    ];
     for (let profile of profiles) {
-      let menuitem = document.createXULElement("menuitem");
+      let menuitem = existingItems.shift();
+      let isNewItem = !menuitem;
+      if (isNewItem) {
+        menuitem = document.createXULElement("menuitem");
+        menuitem.classList.add("menuitem-iconic", "menuitem-iconic-profile");
+        menuitem.setAttribute("command", "Profiles:LaunchProfile");
+      }
       let { themeBg, themeFg } = profile.theme;
       menuitem.setAttribute("profileid", profile.id);
-      menuitem.setAttribute("command", "Profiles:LaunchProfile");
+      menuitem.setAttribute("image", await profile.getAvatarURL(48));
       menuitem.style.setProperty("--menu-profiles-theme-bg", themeBg);
       menuitem.style.setProperty("--menu-profiles-theme-fg", themeFg);
-      menuitem.style.listStyleImage = `url(${await profile.getAvatarURL(48)})`;
-      menuitem.classList.add("menuitem-iconic", "menuitem-iconic-profile");
 
       if (profile.id === currentProfile.id) {
         menuitem.classList.add("current");
@@ -139,31 +139,20 @@ var gProfiles = {
           JSON.stringify({ profileName: profile.name })
         );
       } else {
+        menuitem.classList.remove("current");
+        menuitem.removeAttribute("data-l10n-id");
+        menuitem.removeAttribute("data-l10n-args");
         menuitem.setAttribute("label", profile.name);
       }
 
-      menuPopup.appendChild(menuitem);
+      if (isNewItem) {
+        menuPopup.insertBefore(menuitem, insertionPoint);
+      }
+    }
+    // If there's any old item to remove, do so now.
+    for (let remaining of existingItems) {
+      remaining.remove();
     }
-
-    let newProfile = document.createXULElement("menuitem");
-    newProfile.id = "menu_newProfile";
-    newProfile.className = "menuitem-iconic";
-    newProfile.setAttribute("command", "Profiles:CreateProfile");
-    newProfile.setAttribute("data-l10n-id", "menu-profiles-new-profile");
-    menuPopup.appendChild(newProfile);
-
-    let separator = document.createXULElement("menuseparator");
-    separator.id = "profilesSeparator";
-    menuPopup.appendChild(separator);
-
-    let manageProfiles = document.createXULElement("menuitem");
-    manageProfiles.id = "menu_manageProfiles";
-    manageProfiles.setAttribute("command", "Profiles:ManageProfiles");
-    manageProfiles.setAttribute(
-      "data-l10n-id",
-      "menu-profiles-manage-profiles"
-    );
-    menuPopup.appendChild(manageProfiles);
   },
 
   manageProfiles() {
@@ -180,13 +169,13 @@ var gProfiles = {
     SelectableProfileService.createNewProfile();
   },
 
-  async updateView(target) {
-    await this.populateSubView();
+  updateView(target) {
+    this.populateSubView();
     PanelUI.showSubView("PanelUI-profiles", target);
   },
 
-  async updateFxAView(target) {
-    await this.populateSubView();
+  updateFxAView(target) {
+    this.populateSubView();
     PanelUI.showSubView("PanelUI-profiles", target);
   },
 
@@ -267,6 +256,8 @@ var gProfiles = {
       currentProfile = SelectableProfileService.currentProfile;
     }
 
+    let subview = PanelMultiView.getViewNode(document, "PanelUI-profiles");
+
     let backButton = PanelMultiView.getViewNode(
       document,
       "profiles-appmenu-back-button"
@@ -293,9 +284,62 @@ var gProfiles = {
       "profiles-edit-this-profile-button"
     );
 
+    let profilesList = PanelMultiView.getViewNode(document, "profiles-list");
+    // Automatically created by PanelMultiView.
+    const headerSeparator = profilesHeader.nextElementSibling;
+    let footerSeparator = PanelMultiView.getViewNode(
+      document,
+      "footer-separator"
+    );
+    if (!footerSeparator) {
+      footerSeparator = document.createXULElement("toolbarseparator");
+      footerSeparator.id = "footer-separator";
+    }
+
+    let createProfileButton = PanelMultiView.getViewNode(
+      document,
+      "profiles-create-profile-button"
+    );
+    if (!createProfileButton) {
+      createProfileButton = document.createXULElement("toolbarbutton");
+      createProfileButton.id = "profiles-create-profile-button";
+      createProfileButton.classList.add(
+        "subviewbutton",
+        "subviewbutton-iconic"
+      );
+      createProfileButton.setAttribute(
+        "data-l10n-id",
+        "appmenu-create-profile"
+      );
+    }
+
+    let manageProfilesButton = PanelMultiView.getViewNode(
+      document,
+      "profiles-manage-profiles-button"
+    );
+
+    if (!manageProfilesButton) {
+      manageProfilesButton = document.createXULElement("toolbarbutton");
+      manageProfilesButton.id = "profiles-manage-profiles-button";
+      manageProfilesButton.classList.add(
+        "subviewbutton",
+        "panel-subview-footer-button"
+      );
+      manageProfilesButton.setAttribute(
+        "data-l10n-id",
+        "appmenu-manage-profiles"
+      );
+    }
+
     if (profiles.length < 2) {
       profilesHeader.removeAttribute("style");
       editButton.hidden = true;
+
+      headerSeparator.hidden = false;
+      footerSeparator.hidden = true;
+      const subviewBody = subview.querySelector(".panel-subview-body");
+      subview.insertBefore(createProfileButton, subviewBody);
+      subview.insertBefore(manageProfilesButton, subviewBody);
     } else {
       profilesHeader.style.backgroundColor = "var(--appmenu-profiles-theme-bg)";
       profilesHeader.style.color = "var(--appmenu-profiles-theme-fg)";
@@ -303,11 +347,15 @@ var gProfiles = {
     }
 
     if (currentProfile && profiles.length > 1) {
-      let subview = PanelMultiView.getViewNode(document, "PanelUI-profiles");
       let { themeBg, themeFg } = currentProfile.theme;
       subview.style.setProperty("--appmenu-profiles-theme-bg", themeBg);
       subview.style.setProperty("--appmenu-profiles-theme-fg", themeFg);
 
+      headerSeparator.hidden = true;
+      subview.appendChild(footerSeparator);
+      subview.appendChild(createProfileButton);
+      subview.appendChild(manageProfilesButton);
+
       let headerText = PanelMultiView.getViewNode(
         document,
         "profiles-header-content"
@@ -333,7 +381,6 @@ var gProfiles = {
     let subtitle = PanelMultiView.getViewNode(document, "profiles-subtitle");
     subtitle.hidden = profiles.length < 2;
 
-    let profilesList = PanelMultiView.getViewNode(document, "profiles-list");
     while (profilesList.lastElementChild) {
       profilesList.lastElementChild.remove();
     }
@@ -349,7 +396,7 @@ var gProfiles = {
       let { themeFg, themeBg } = profile.theme;
       button.style.setProperty("--appmenu-profiles-theme-bg", themeBg);
       button.style.setProperty("--appmenu-profiles-theme-fg", themeFg);
-      button.setAttribute("image", `${await profile.getAvatarURL(16)}`);
+      button.setAttribute("image", await profile.getAvatarURL(16));
 
       profilesList.appendChild(button);
     }
