@@ -3,11 +3,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* eslint-env mozilla/browser-window */
-
 document.addEventListener(
   "DOMContentLoaded",
   () => {
+    const lazy = {};
+    ChromeUtils.defineESModuleGetters(lazy, {
+      TabMetrics: "moz-src:///browser/components/tabbrowser/TabMetrics.sys.mjs",
+    });
     let mainPopupSet = document.getElementById("mainPopupSet");
     // eslint-disable-next-line complexity
     mainPopupSet.addEventListener("command", event => {
@@ -45,7 +47,9 @@ document.addEventListener(
           );
           break;
         case "context_pinTab":
-          gBrowser.pinTab(TabContextMenu.contextTab);
+          gBrowser.pinTab(TabContextMenu.contextTab, {
+            telemetrySource: lazy.TabMetrics.METRIC_SOURCE.TAB_MENU,
+          });
           break;
         case "context_unpinTab":
           gBrowser.unpinTab(TabContextMenu.contextTab);
@@ -84,16 +88,31 @@ document.addEventListener(
           TabContextMenu.closeContextTabs();
           break;
         case "context_closeDuplicateTabs":
-          gBrowser.removeDuplicateTabs(TabContextMenu.contextTab);
+          gBrowser.removeDuplicateTabs(
+            TabContextMenu.contextTab,
+            lazy.TabMetrics.userTriggeredContext()
+          );
           break;
         case "context_closeTabsToTheStart":
-          gBrowser.removeTabsToTheStartFrom(TabContextMenu.contextTab);
+          gBrowser.removeTabsToTheStartFrom(
+            TabContextMenu.contextTab,
+            lazy.TabMetrics.userTriggeredContext()
+          );
           break;
         case "context_closeTabsToTheEnd":
-          gBrowser.removeTabsToTheEndFrom(TabContextMenu.contextTab);
+          gBrowser.removeTabsToTheEndFrom(
+            TabContextMenu.contextTab,
+            lazy.TabMetrics.userTriggeredContext()
+          );
           break;
         case "context_closeOtherTabs":
-          gBrowser.removeAllTabsBut(TabContextMenu.contextTab);
+          gBrowser.removeAllTabsBut(
+            TabContextMenu.contextTab,
+            lazy.TabMetrics.userTriggeredContext()
+          );
+          break;
+        case "context_unloadTab":
+          TabContextMenu.explicitUnloadTabs();
           break;
         case "context_fullscreenAutohide":
           FullScreen.setAutohide();
@@ -102,6 +121,64 @@ document.addEventListener(
           BrowserCommands.fullScreen();
           break;
 
+        // == open-tab-group-context-menu ==
+        case "open-tab-group-context-menu_moveToNewWindow":
+          {
+            let { tabGroupId } = event.target.parentElement.triggerNode.dataset;
+            let tabGroup = gBrowser.getTabGroupById(tabGroupId);
+            tabGroup.ownerGlobal.gBrowser.replaceGroupWithWindow(tabGroup);
+          }
+          break;
+        case "open-tab-group-context-menu_moveToThisWindow":
+          {
+            let { tabGroupId } = event.target.parentElement.triggerNode.dataset;
+            let otherTabGroup = gBrowser.getTabGroupById(tabGroupId);
+            let adoptedTabGroup = gBrowser.adoptTabGroup(otherTabGroup, {
+              tabIndex: gBrowser.tabs.length,
+            });
+            adoptedTabGroup.select();
+          }
+          break;
+        case "open-tab-group-context-menu_delete":
+          {
+            let { tabGroupId } = event.target.parentElement.triggerNode.dataset;
+            let tabGroup = gBrowser.getTabGroupById(tabGroupId);
+            // Tabs need to be removed by their owning `Tabbrowser` or else
+            // there are errors.
+            tabGroup.ownerGlobal.gBrowser.removeTabGroup(
+              tabGroup,
+              lazy.TabMetrics.userTriggeredContext(
+                lazy.TabMetrics.METRIC_SOURCE.TAB_OVERFLOW_MENU
+              )
+            );
+          }
+          break;
+
+        // == saved-tab-group-context-menu ==
+        case "saved-tab-group-context-menu_openInThisWindow":
+          {
+            let { tabGroupId } = event.target.parentElement.triggerNode.dataset;
+            SessionStore.openSavedTabGroup(tabGroupId, window, {
+              source: lazy.TabMetrics.METRIC_SOURCE.TAB_OVERFLOW_MENU,
+            });
+          }
+          break;
+        case "saved-tab-group-context-menu_openInNewWindow":
+          {
+            // TODO Bug 1940112: "Open Group in New Window" should directly restore saved tab groups into a new window
+            let { tabGroupId } = event.target.parentElement.triggerNode.dataset;
+            let tabGroup = SessionStore.openSavedTabGroup(tabGroupId, window, {
+              source: lazy.TabMetrics.METRIC_SOURCE.TAB_OVERFLOW_MENU,
+            });
+            gBrowser.replaceGroupWithWindow(tabGroup);
+          }
+          break;
+        case "saved-tab-group-context-menu_delete":
+          {
+            let { tabGroupId } = event.target.parentElement.triggerNode.dataset;
+            SessionStore.forgetSavedTabGroup(tabGroupId);
+          }
+          break;
         // == editBookmarkPanel ==
         case "editBookmarkPanel_showForNewBookmarks":
           StarUI.onShowForNewBookmarksCheckboxCommand();
@@ -127,7 +204,8 @@ document.addEventListener(
           SidebarController.reversePosition();
           break;
         case "sidebar-menu-close":
-          SidebarController.hide();
+          // Close the sidebar UI, but leave it otherwise in its current state
+          SidebarController.hide({ dismissPanel: false });
           break;
 
         // == toolbar-context-menu ==
@@ -156,7 +234,20 @@ document.addEventListener(
         case "toolbar-context-autohide-downloads-button":
           ToolbarContextMenu.onDownloadsAutoHideChange(event);
           break;
+        case "toolbar-context-always-show-extensions-button":
+          if (event.target.getAttribute("checked") == "true") {
+            gUnifiedExtensions.showExtensionsButtonInToolbar();
+          } else {
+            gUnifiedExtensions.hideExtensionsButtonFromToolbar();
+          }
+          break;
         case "toolbar-context-remove-from-toolbar":
+          if (
+            event.target.parentNode.triggerNode === gUnifiedExtensions.button
+          ) {
+            gUnifiedExtensions.hideExtensionsButtonFromToolbar();
+            break;
+          }
           gCustomizeMode.removeFromArea(
             event.target.parentNode.triggerNode,
             "toolbar-context-menu"
@@ -182,6 +273,15 @@ document.addEventListener(
         case "toolbar-context-selectAllTabs":
           gBrowser.selectAllTabs();
           break;
+        case "toolbar-context-customize":
+          gCustomizeMode.enter();
+          break;
+        case "toolbar-context-toggle-vertical-tabs":
+          SidebarController.toggleVerticalTabs();
+          break;
+        case "toolbar-context-customize-sidebar":
+          SidebarController.show("viewCustomizeSidebar");
+          break;
         case "toolbar-context-full-screen-autohide":
           FullScreen.setAutohide();
           break;
@@ -301,9 +401,24 @@ document.addEventListener(
             "panelitem-context"
           );
           break;
+
+        // == sharing-tabs-warning-panel ==
+        case "sharing-warning-proceed-to-tab":
+          gSharedTabWarning.allowSharedTabSwitch();
+          break;
       }
     });
 
+    const containerHistoryPopup = document.getElementById(
+      "sidebar-history-context-menu-container-popup"
+    );
+    containerHistoryPopup.addEventListener("command", event =>
+      PlacesUIUtils.openInContainerTab(event)
+    );
+    containerHistoryPopup.addEventListener("popupshowing", event =>
+      PlacesUIUtils.createContainerTabMenu(event)
+    );
+
     document
       .getElementById("context_reopenInContainerPopupMenu")
       .addEventListener("command", event => {
@@ -321,11 +436,13 @@ document.addEventListener(
 
         const tabGroupId = event.target.getAttribute("tab-group-id");
         const group = gBrowser.getTabGroupById(tabGroupId);
-        if (!group) {
-          return;
+        if (group) {
+          TabContextMenu.moveTabsToGroup(group);
         }
 
-        TabContextMenu.moveTabsToGroup(group);
+        if (SessionStore.getSavedTabGroup(tabGroupId)) {
+          TabContextMenu.addTabsToSavedGroup(tabGroupId);
+        }
       });
 
     document
@@ -344,6 +461,12 @@ document.addEventListener(
         gUnifiedExtensions.onContextMenuCommand(event.currentTarget, event);
       });
 
+    document
+      .getElementById("webRTC-selectWindow-menulist")
+      .addEventListener("command", event => {
+        webrtcUI.updateWarningLabel(event.currentTarget);
+      });
+
     mainPopupSet.addEventListener("popupshowing", event => {
       switch (event.target.id) {
         case "context_sendTabToDevicePopupMenu":
@@ -370,7 +493,11 @@ document.addEventListener(
           );
           ToolbarContextMenu.updateDownloadsAutoHide(event.target);
           ToolbarContextMenu.updateDownloadsAlwaysOpenPanel(event.target);
-          ToolbarContextMenu.updateExtension(event.target, event);
+          ToolbarContextMenu.updateExtensionsButtonContextMenu(event.target);
+          ToolbarContextMenu.updateExtension(event.target);
+          // hideLeadingSeparatorIfNeeded must be called last after updating the menu items above,
+          // as they may change which items are visible.
+          ToolbarContextMenu.hideLeadingSeparatorIfNeeded(event.target);
           break;
         case "pageActionContextMenu":
           BrowserPageActions.onContextMenuShowing(event, event.target);
@@ -379,7 +506,7 @@ document.addEventListener(
           gBrowser.createTooltip(event);
           break;
         case "dynamic-shortcut-tooltip":
-          UpdateDynamicShortcutTooltipText(event.target);
+          DynamicShortcutTooltip.updateText(event.target);
           break;
         case "SyncedTabsOpenSelectedInContainerTabMenu":
           createUserContextMenu(event, { isContextMenu: true });
@@ -391,6 +518,9 @@ document.addEventListener(
           gCustomizeMode.onPanelContextMenuShowing(event);
           ToolbarContextMenu.updateExtension(event.target);
           break;
+        case "bhTooltip":
+          BookmarksEventHandler.fillInBHTooltip(event.target, event);
+          break;
       }
     });
 
@@ -402,6 +532,32 @@ document.addEventListener(
         }
       });
 
+    // Enable/disable some `open-tab-group-context-menu` options based on the
+    // specific tab group context.
+    document
+      .getElementById("open-tab-group-context-menu")
+      .addEventListener("popupshowing", event => {
+        if (event.target.id == "open-tab-group-context-menu") {
+          // Disable "Move Group to This Window" menu option for tab groups
+          // that are open in the current window.
+          let { tabGroupId } = event.target.triggerNode.dataset;
+          let tabGroup = gBrowser.getTabGroupById(tabGroupId);
+          let tabGroupIsInThisWindow = tabGroup.ownerDocument == document;
+          event.target.querySelector(
+            "#open-tab-group-context-menu_moveToThisWindow"
+          ).disabled = tabGroupIsInThisWindow;
+
+          // Disable "Move Group to New Window" menu option for tab groups
+          // that are the only things in their respective window.
+          let groupAloneInWindow =
+            tabGroup.tabs.length ==
+            tabGroup.ownerGlobal.gBrowser.openTabs.length;
+          event.target.querySelector(
+            "#open-tab-group-context-menu_moveToNewWindow"
+          ).disabled = groupAloneInWindow;
+        }
+      });
+
     mainPopupSet.addEventListener("popupshown", event => {
       switch (event.target.id) {
         case "sharing-tabs-warning-panel":
@@ -416,6 +572,7 @@ document.addEventListener(
     mainPopupSet.addEventListener("popuphiding", event => {
       switch (event.target.id) {
         case "tabbrowser-tab-tooltip":
+        case "bhTooltip":
           event.target.removeAttribute("position");
           break;
       }
@@ -423,9 +580,6 @@ document.addEventListener(
 
     mainPopupSet.addEventListener("popuphidden", event => {
       switch (event.target.id) {
-        case "tabContextMenu":
-          TabContextMenu.contextTab = null;
-          break;
         case "full-page-translations-panel-settings-menupopup":
           FullPageTranslationsPanel.handleSettingsPopupHiddenEvent();
           break;
