@@ -21,6 +21,11 @@ const SHOW_GLOBAL_MUTE_TOGGLES = Services.prefs.getBoolPref(
   false
 );
 
+const SHOW_ALWAYS_ASK = Services.prefs.getBoolPref(
+  "permissions.media.show_always_ask.enabled",
+  false
+);
+
 let IsIndicatorDisabled =
   AppConstants.isPlatformAndVersionAtLeast("macosx", 14.0) &&
   !Services.prefs.getBoolPref(
@@ -61,7 +66,7 @@ function whenDelayedStartupFinished(aWindow) {
 }
 
 function promiseIndicatorWindow() {
-  let startTime = performance.now();
+  let startTime = ChromeUtils.now();
 
   return new Promise(resolve => {
     Services.obs.addObserver(function obs(win) {
@@ -339,7 +344,7 @@ function promiseMessage(
   aCount = 1,
   browser = gBrowser.selectedBrowser
 ) {
-  let startTime = performance.now();
+  let startTime = ChromeUtils.now();
   let promise = ContentTask.spawn(
     browser,
     [aMessage, aCount],
@@ -372,7 +377,7 @@ function promiseMessage(
 }
 
 function promisePopupNotificationShown(aName, aAction, aWindow = window) {
-  let startTime = performance.now();
+  let startTime = ChromeUtils.now();
   return new Promise(resolve => {
     aWindow.PopupNotifications.panel.addEventListener(
       "popupshown",
@@ -453,7 +458,7 @@ async function activateSecondaryAction(aAction) {
 }
 
 async function getMediaCaptureState() {
-  let startTime = performance.now();
+  let startTime = ChromeUtils.now();
 
   function gatherBrowsingContexts(aBrowsingContext) {
     let list = [aBrowsingContext];
@@ -660,18 +665,53 @@ async function getBrowsingContextsAndFrameIdsForSubFrames(
   return browsingContextsAndFrames;
 }
 
+/**
+ * Test helper for getUserMedia calls.
+ * @param {boolean} aRequestAudio - Whether to request audio
+ * @param {boolean} aRequestVideo - Whether to request video
+ * @param {string} aFrameId - The ID of the frame
+ * @param {string} aType - The type of screen sharing.
+ * @param {BrowsingContext} aBrowsingContext - The browsing context
+ * @param {boolean} [aBadDevice=false] - Whether to use a bad device
+ * @param {boolean} [viaButtonClick=false] - Whether to call gUM directly or to
+ *   request via simulated button click.
+ * @returns {Promise} - Resolves when the gUM request has been made.
+ */
 async function promiseRequestDevice(
   aRequestAudio,
   aRequestVideo,
   aFrameId,
   aType,
   aBrowsingContext,
-  aBadDevice = false
+  aBadDevice = false,
+  viaButtonClick = false
 ) {
   info("requesting devices");
   let bc =
     aBrowsingContext ??
     (await getBrowsingContextForFrame(gBrowser.selectedBrowser, aFrameId));
+
+  if (viaButtonClick) {
+    return SpecialPowers.spawn(
+      bc,
+      [{ aRequestAudio, aRequestVideo, aType, aBadDevice }],
+      async function (args) {
+        let global = content.wrappedJSObject;
+        global.queueRequestDeviceViaBtn(
+          args.aRequestAudio,
+          args.aRequestVideo,
+          args.aType,
+          args.aBadDevice
+        );
+        await EventUtils.synthesizeMouseAtCenter(
+          global.document.getElementById("gum"),
+          {},
+          content
+        );
+      }
+    );
+  }
+
   return SpecialPowers.spawn(
     bc,
     [{ aRequestAudio, aRequestVideo, aType, aBadDevice }],
@@ -681,7 +721,8 @@ async function promiseRequestDevice(
         args.aRequestAudio,
         args.aRequestVideo,
         args.aType,
-        args.aBadDevice
+        args.aBadDevice,
+        args.withUserActivation
       );
     }
   );
@@ -916,18 +957,28 @@ async function checkSharingUI(
   // First check the icon above the control center (i) icon.
   let permissionBox = doc.getElementById("identity-permission-box");
   let webrtcSharingIcon = doc.getElementById("webrtc-sharing-icon");
-  ok(webrtcSharingIcon.hasAttribute("sharing"), "sharing attribute is set");
+  let expectOn = aExpected.audio || aExpected.video || aExpected.screen;
+  if (expectOn) {
+    ok(webrtcSharingIcon.hasAttribute("sharing"), "sharing attribute is set");
+  } else {
+    ok(
+      !webrtcSharingIcon.hasAttribute("sharing"),
+      "sharing attribute is not set"
+    );
+  }
   let sharing = webrtcSharingIcon.getAttribute("sharing");
-  if (aExpected.screen && !IsIndicatorDisabled) {
-    is(sharing, "screen", "showing screen icon in the identity block");
-  } else if (aExpected.video == STATE_CAPTURE_ENABLED && !IsIndicatorDisabled) {
-    is(sharing, "camera", "showing camera icon in the identity block");
-  } else if (aExpected.audio == STATE_CAPTURE_ENABLED && !IsIndicatorDisabled) {
-    is(sharing, "microphone", "showing mic icon in the identity block");
-  } else if (aExpected.video && !IsIndicatorDisabled) {
-    is(sharing, "camera", "showing camera icon in the identity block");
-  } else if (aExpected.audio && !IsIndicatorDisabled) {
-    is(sharing, "microphone", "showing mic icon in the identity block");
+  if (!IsIndicatorDisabled) {
+    if (aExpected.screen) {
+      is(sharing, "screen", "showing screen icon in the identity block");
+    } else if (aExpected.video == STATE_CAPTURE_ENABLED) {
+      is(sharing, "camera", "showing camera icon in the identity block");
+    } else if (aExpected.audio == STATE_CAPTURE_ENABLED) {
+      is(sharing, "microphone", "showing mic icon in the identity block");
+    } else if (aExpected.video) {
+      is(sharing, "camera", "showing camera icon in the identity block");
+    } else if (aExpected.audio) {
+      is(sharing, "microphone", "showing mic icon in the identity block");
+    }
   }
 
   let allStreamsPaused = Object.values(aExpected).every(isPaused);
@@ -1003,6 +1054,13 @@ async function checkSharingUI(
         true,
         "should not show " + id + " state label in the permission panel"
       );
+      if (state != SitePermissions.PROMPT || SHOW_ALWAYS_ASK) {
+        isnot(
+          scope,
+          SitePermissions.SCOPE_PERSISTENT,
+          "persistent permission not shown"
+        );
+      }
     } else {
       // This will happen if there are persistent permissions set.
       ok(
@@ -1011,6 +1069,21 @@ async function checkSharingUI(
       );
       is(item.length, 1, "should not show more than 1 " + id + " item");
       is(icon.length, 1, "should not show more than 1 " + id + " icon");
+
+      // Note: To pass, this one needs state and/or scope passed into
+      // checkSharingUI() for values other than ALLOW and SCOPE_TEMPORARY
+      is(
+        stateLabel?.textContent,
+        SitePermissions.getCurrentStateLabel(state, id, scope),
+        "should show correct item label for " + id
+      );
+      if (!SHOW_ALWAYS_ASK) {
+        isnot(
+          state,
+          state == SitePermissions.PROMPT,
+          "always ask permission should be hidden"
+        );
+      }
     }
   }
   aWin.gPermissionPanel._permissionPopup.hidePopup();
@@ -1020,7 +1093,9 @@ async function checkSharingUI(
   );
 
   // Check the global indicators.
-  await assertWebRTCIndicatorStatus(aExpectedGlobal || aExpected);
+  if (expectOn) {
+    await assertWebRTCIndicatorStatus(aExpectedGlobal || aExpected);
+  }
 }
 
 async function checkNotSharing() {
@@ -1192,7 +1267,7 @@ async function runTests(tests, options = {}) {
   gObserveSubFrames = SpecialPowers.useRemoteSubframes ? options.subFrames : {};
 
   for (let testCase of tests) {
-    let startTime = performance.now();
+    let startTime = ChromeUtils.now();
     info(testCase.desc);
     if (
       !testCase.skipObserverVerification &&
