@@ -128,7 +128,7 @@ export const MultiStageAboutWelcome = props => {
   }, [transition]);
 
   // Transition to next screen, opening about:home on last screen button CTA
-  const handleTransition = () => {
+  const handleTransition = goBack => {
     // Only handle transitioning out from a screen once.
     if (transition === "out") {
       return;
@@ -140,7 +140,10 @@ export const MultiStageAboutWelcome = props => {
     // Actually move forwards after all transitions finish.
     setTimeout(
       () => {
-        if (index < screens.length - 1) {
+        if (goBack) {
+          setTransition(props.transitions ? "in" : "");
+          setScreenIndex(prevState => prevState - 1);
+        } else if (index < screens.length - 1) {
           setTransition(props.transitions ? "in" : "");
           setScreenIndex(prevState => prevState + 1);
         } else {
@@ -199,11 +202,12 @@ export const MultiStageAboutWelcome = props => {
   // multi select screen.
   const [activeMultiSelects, setActiveMultiSelects] = useState({});
 
-  // Save the active single select state for each screen as string value keyed
+  // Save the active single select state for each screen as an object keyed
   // by screen id. Similar to above, this allows us to remember the state of
   // each screen's single select picker when navigating back and forth between
-  // screens.
-  const [activeSingleSelects, setActiveSingleSelects] = useState({});
+  // screens, and allows us to have multiple single selects on a screen.
+  const [activeSingleSelectSelections, setActiveSingleSelectSelections] =
+    useState({});
 
   // Get the active theme so the rendering code can make it selected
   // by default.
@@ -283,14 +287,25 @@ export const MultiStageAboutWelcome = props => {
             });
           };
 
-          const setActiveSingleSelect = valueOrFn =>
-            setActiveSingleSelects(prevState => ({
-              ...prevState,
-              [currentScreen.id]:
-                typeof valueOrFn === "function"
-                  ? valueOrFn(prevState[currentScreen.id])
-                  : valueOrFn,
-            }));
+          const setActiveSingleSelectSelection = (
+            valueOrFn,
+            singleSelectId
+          ) => {
+            setActiveSingleSelectSelections(prevState => {
+              const currentScreenSelections = prevState[currentScreen.id] || {};
+
+              return {
+                ...prevState,
+                [currentScreen.id]: {
+                  ...currentScreenSelections,
+                  [singleSelectId]:
+                    typeof valueOrFn === "function"
+                      ? valueOrFn(prevState[currentScreen.id])
+                      : valueOrFn,
+                },
+              };
+            });
+          };
 
           return index === order ? (
             <WelcomeScreen
@@ -316,8 +331,10 @@ export const MultiStageAboutWelcome = props => {
               activeMultiSelect={activeMultiSelects[currentScreen.id]}
               setActiveMultiSelect={setActiveMultiSelect}
               autoAdvance={currentScreen.auto_advance}
-              activeSingleSelect={activeSingleSelects[currentScreen.id]}
-              setActiveSingleSelect={setActiveSingleSelect}
+              activeSingleSelectSelections={
+                activeSingleSelectSelections[currentScreen.id]
+              }
+              setActiveSingleSelectSelection={setActiveSingleSelectSelection}
               negotiatedLanguage={negotiatedLanguage}
               langPackInstallPhase={langPackInstallPhase}
               forceHideStepsIndicator={currentScreen.force_hide_steps_indicator}
@@ -327,6 +344,13 @@ export const MultiStageAboutWelcome = props => {
               }
               installedAddons={installedAddons}
               setInstalledAddons={setInstalledAddons}
+              addonId={props.addonId}
+              addonType={props.addonType}
+              addonName={props.addonName}
+              addonURL={props.addonURL}
+              addonIconURL={props.addonIconURL}
+              themeScreenshots={props.themeScreenshots}
+              isRtamo={currentScreen.content.isRtamo}
             />
           ) : null;
         })}
@@ -335,81 +359,170 @@ export const MultiStageAboutWelcome = props => {
   );
 };
 
-export const SecondaryCTA = props => {
-  const targetElement = props.position
-    ? `secondary_button_${props.position}`
-    : `secondary_button`;
-  let buttonStyling = props.content.secondary_button?.has_arrow_icon
+const renderSingleSecondaryCTAButton = ({
+  content,
+  button,
+  targetElement,
+  position,
+  handleAction,
+  activeMultiSelect,
+  isArrayItem,
+  index = null,
+}) => {
+  let buttonStyling = button?.has_arrow_icon
     ? `secondary arrow-icon`
     : `secondary`;
-  const isPrimary = props.content.secondary_button?.style === "primary";
+  const isPrimary = button?.style === "primary";
   const isTextLink =
-    !["split", "callout"].includes(props.content.position) &&
-    props.content.tiles?.type !== "addons-picker" &&
+    !["split", "callout"].includes(content.position) &&
+    content.tiles?.type !== "addons-picker" &&
     !isPrimary;
-  const isSplitButton =
-    props.content.submenu_button?.attached_to === targetElement;
+  const isSplitButton = content.submenu_button?.attached_to === targetElement;
+
   let className = "secondary-cta";
-  if (props.position) {
-    className += ` ${props.position}`;
+  if (position) {
+    className += ` ${position}`;
   }
   if (isSplitButton) {
     className += " split-button-container";
   }
-  const isDisabled = React.useCallback(
-    disabledValue => {
-      if (disabledValue === "hasActiveMultiSelect") {
-        if (!props.activeMultiSelect) {
-          return true;
-        }
-
-        for (const key in props.activeMultiSelect) {
-          if (props.activeMultiSelect[key]?.length > 0) {
-            return false;
-          }
-        }
 
+  const computeDisabled = disabledValue => {
+    if (disabledValue === "hasActiveMultiSelect") {
+      if (!activeMultiSelect) {
         return true;
       }
 
-      return disabledValue;
-    },
-    [props.activeMultiSelect]
-  );
+      for (const key in activeMultiSelect) {
+        if (activeMultiSelect[key]?.length > 0) {
+          return false;
+        }
+      }
+
+      return true;
+    }
+    return disabledValue;
+  };
 
   if (isTextLink) {
     buttonStyling += " text-link";
   }
 
   if (isPrimary) {
-    buttonStyling = props.content.secondary_button?.has_arrow_icon
-      ? `primary arrow-icon`
-      : `primary`;
+    buttonStyling = button?.has_arrow_icon ? `primary arrow-icon` : `primary`;
   }
 
+  // We have to provide handleAction with the expected action here,
+  // since the data doesn't actually exist in JSON content
+  const shimmedHandleAction = event => {
+    if (isArrayItem && button?.action) {
+      return handleAction(event, button.action);
+    }
+    return handleAction(event);
+  };
+
+  let buttonId = "secondary_button";
+  buttonId += index !== null ? `_${index}` : "";
+
   return (
-    <div className={className}>
-      <Localized text={props.content[targetElement].text}>
+    <div className={className} key={targetElement}>
+      <Localized text={button?.text}>
         <span />
       </Localized>
-      <Localized text={props.content[targetElement].label}>
+      <Localized text={button?.label}>
         <button
+          id={buttonId}
           className={buttonStyling}
           value={targetElement}
-          disabled={isDisabled(props.content.secondary_button?.disabled)}
-          onClick={props.handleAction}
+          disabled={computeDisabled(button?.disabled)}
+          onClick={shimmedHandleAction}
         />
       </Localized>
       {isSplitButton ? (
-        <SubmenuButton
-          content={props.content}
-          handleAction={props.handleAction}
-        />
+        <SubmenuButton content={content} handleAction={handleAction} />
       ) : null}
     </div>
   );
 };
 
+export const SecondaryCTA = props => {
+  const { content, position } = props;
+
+  const targetElement = position
+    ? `secondary_button_${position}`
+    : "secondary_button";
+  const buttonData = content[targetElement];
+
+  if (!buttonData) {
+    return null;
+  }
+
+  const buttons = React.useMemo(
+    () => (Array.isArray(buttonData) ? buttonData : [buttonData]),
+    [buttonData]
+  );
+
+  const [visibleButtons, setVisibleButtons] = React.useState([]);
+
+  React.useEffect(() => {
+    (async () => {
+      const filteredButtons = [];
+      for (const button of buttons) {
+        // No targeting, show by default for backwards compatibility
+        if (!button?.targeting) {
+          filteredButtons.push(button);
+          continue;
+        }
+
+        try {
+          const shouldShowButton = await window.AWEvaluateAttributeTargeting(
+            button.targeting
+          );
+          if (shouldShowButton) {
+            filteredButtons.push(button);
+          }
+        } catch (e) {
+          console.error("SecondaryCTA targeting failed:", button.targeting, e);
+        }
+      }
+      setVisibleButtons(filteredButtons);
+    })();
+  }, [buttons]);
+
+  if (!visibleButtons.length) {
+    return null;
+  }
+
+  if (Array.isArray(buttonData)) {
+    return (
+      <div className="secondary-buttons-top-container">
+        {visibleButtons.map((button, index) =>
+          renderSingleSecondaryCTAButton({
+            content,
+            button,
+            targetElement: `${targetElement}_${index}`,
+            position,
+            handleAction: props.handleAction,
+            activeMultiSelect: props.activeMultiSelect,
+            isArrayItem: true,
+            index,
+          })
+        )}
+      </div>
+    );
+  }
+
+  return renderSingleSecondaryCTAButton({
+    content,
+    button: visibleButtons[0],
+    targetElement,
+    position,
+    handleAction: props.handleAction,
+    activeMultiSelect: props.activeMultiSelect,
+    isArrayItem: false,
+  });
+};
+
 export const StepsIndicator = props => {
   let steps = [];
   for (let i = 0; i < props.totalNumberOfScreens; i++) {
@@ -460,7 +573,10 @@ export class WelcomeScreen extends React.PureComponent {
           ...flowParams,
         };
       }
-      data = { ...data, extraParams: params };
+      data = {
+        ...data,
+        extraParams: { ...params, ...data?.extraParams },
+      };
     } else if (type === "OPEN_URL") {
       let url = new URL(data.args);
       addUtmParams(url, `${UTMTerm}-screen`);
@@ -474,24 +590,7 @@ export class WelcomeScreen extends React.PureComponent {
     return AboutWelcomeUtils.handleUserAction({ type, data });
   }
 
-  async handleAction(event) {
-    let { props } = this;
-    const value =
-      event.currentTarget.value ?? event.currentTarget.getAttribute("value");
-    const source = event.source || value;
-    let targetContent =
-      props.content[value] ||
-      props.content.tiles ||
-      props.content.languageSwitcher;
-
-    if (value === "submenu_button" && event.action) {
-      targetContent = { action: event.action };
-    }
-
-    if (!(targetContent && targetContent.action)) {
-      return;
-    }
-    // Send telemetry before waiting on actions
+  logTelemetry({ value, event, source, props }) {
     AboutWelcomeUtils.sendActionTelemetry(props.messageId, source, event.name);
 
     // Send additional telemetry if a messaging surface like feature callout is
@@ -500,8 +599,98 @@ export class WelcomeScreen extends React.PureComponent {
     if (value === "dismiss_button" && !event.name) {
       AboutWelcomeUtils.sendDismissTelemetry(props.messageId, source);
     }
+  }
+
+  async handleMigrationIfNeeded(action, props) {
+    const hasMigrate = a =>
+      a.type === "SHOW_MIGRATION_WIZARD" ||
+      (a.type === "MULTI_ACTION" && a.data?.actions?.some(hasMigrate));
+
+    if (hasMigrate(action)) {
+      await window.AWWaitForMigrationClose();
+      AboutWelcomeUtils.sendActionTelemetry(props.messageId, "migrate_close");
+    }
+  }
+
+  applyThemeIfNeeded(action, event) {
+    if (!action.theme) {
+      return;
+    }
+
+    const themeToUse =
+      action.theme === "<event>"
+        ? event.currentTarget.value
+        : this.props.initialTheme || action.theme;
+
+    this.props.setActiveTheme(themeToUse);
+    window.AWSelectTheme(themeToUse);
+  }
+
+  handlePickerAction(value) {
+    const tileGroups = Array.isArray(this.props.content.tiles)
+      ? this.props.content.tiles
+      : [this.props.content.tiles];
+
+    for (const tile of tileGroups) {
+      if (!tile?.data) {
+        continue;
+      }
+
+      for (const opt of tile.data) {
+        if (opt.id === value) {
+          AboutWelcomeUtils.handleUserAction(opt.action);
+          return;
+        }
+      }
+    }
+  }
+
+  resolveActionFromContent(value, event, props) {
+    if (
+      (value === "submenu_button" || value === "tile_button") &&
+      event.action
+    ) {
+      return event.action;
+    }
+
+    const { content } = props;
+    const targetContent =
+      content[value] || content.tiles || content.languageSwitcher;
+
+    if (!targetContent) {
+      return null;
+    }
+
+    if (Array.isArray(targetContent)) {
+      for (const tile of targetContent) {
+        const matchedTile = tile.data.find(t => t.id === value);
+        if (matchedTile?.action) {
+          return matchedTile.action;
+        }
+      }
+      return null;
+    }
+
+    return targetContent.action ?? null;
+  }
+
+  async handleAction(event, providedAction = null) {
+    const { props } = this;
+    const value =
+      event.currentTarget.value ?? event.currentTarget.getAttribute("value");
+    const source = event.source || value;
+
+    let action =
+      providedAction || this.resolveActionFromContent(value, event, props);
+
+    if (!action) {
+      console.error("Failed to resolve action");
+      return;
+    }
+
+    // Send telemetry before waiting on actions
+    this.logTelemetry({ value, event, source, props });
 
-    let { action } = targetContent;
     action = JSON.parse(JSON.stringify(action));
 
     if (action.collectSelect) {
@@ -510,15 +699,12 @@ export class WelcomeScreen extends React.PureComponent {
 
     let actionResult;
     if (["OPEN_URL", "SHOW_FIREFOX_ACCOUNTS"].includes(action.type)) {
-      actionResult = this.handleOpenURL(
-        action,
-        props.flowParams,
-        props.UTMTerm
-      );
+      this.handleOpenURL(action, props.flowParams, props.UTMTerm);
     } else if (action.type) {
-      actionResult = action.needsAwait
-        ? await AboutWelcomeUtils.handleUserAction(action)
-        : AboutWelcomeUtils.handleUserAction(action);
+      let actionPromise = AboutWelcomeUtils.handleUserAction(action);
+      if (action.needsAwait) {
+        actionResult = await actionPromise;
+      }
       if (action.type === "FXA_SIGNIN_FLOW") {
         AboutWelcomeUtils.sendActionTelemetry(
           props.messageId,
@@ -526,33 +712,26 @@ export class WelcomeScreen extends React.PureComponent {
           "FXA_SIGNIN_FLOW"
         );
       }
-      // Wait until migration closes to complete the action
-      const hasMigrate = a =>
-        a.type === "SHOW_MIGRATION_WIZARD" ||
-        (a.type === "MULTI_ACTION" && a.data?.actions?.some(hasMigrate));
-      if (hasMigrate(action)) {
-        await window.AWWaitForMigrationClose();
-        AboutWelcomeUtils.sendActionTelemetry(props.messageId, "migrate_close");
+
+      if (action.type === "INSTALL_ADDON_FROM_URL") {
+        const url = props.addonURL;
+        if (!action.data) {
+          return;
+        }
+        // Set add-on url in action.data.url property from JSON
+        action.data = { ...action.data, url };
+
+        AboutWelcomeUtils.handleUserAction(action);
       }
+      // Wait until migration closes to complete the action
+      await this.handleMigrationIfNeeded(action, props);
     }
 
     // A special tiles.action.theme value indicates we should use the event's value vs provided value.
-    if (action.theme) {
-      let themeToUse =
-        action.theme === "<event>"
-          ? event.currentTarget.value
-          : this.props.initialTheme || action.theme;
-      this.props.setActiveTheme(themeToUse);
-      window.AWSelectTheme(themeToUse);
-    }
+    this.applyThemeIfNeeded(action, event);
 
     if (action.picker) {
-      let options = props.content.tiles.data;
-      options.forEach(opt => {
-        if (opt.id === value) {
-          AboutWelcomeUtils.handleUserAction(opt.action);
-        }
-      });
+      this.handlePickerAction(value);
     }
 
     // If the action has persistActiveTheme: true, we set the initial theme to the currently active theme
@@ -561,7 +740,7 @@ export class WelcomeScreen extends React.PureComponent {
       this.props.setInitialTheme(this.props.activeTheme);
     }
 
-    // `navigate` and `dismiss` can be true/false/undefined, or they can be a
+    // `navigate`, `goBack` and `dismiss` can be true/false/undefined, or they can be a
     // string "actionResult" in which case we should use the actionResult
     // (boolean resolved by handleUserAction)
     const shouldDoBehavior = behavior => {
@@ -580,7 +759,16 @@ export class WelcomeScreen extends React.PureComponent {
     };
 
     if (shouldDoBehavior(action.navigate)) {
-      props.navigate();
+      props.navigate(action.goBack);
+    }
+
+    // Used by FeatureCallout to advance screens by re-rendering the whole
+    // wrapper, updating anchor, page_event_listeners, etc. `navigate` only
+    // updates the inner content. Only implemented by FeatureCallout.
+    if (action.advance_screens) {
+      if (shouldDoBehavior(action.advance_screens.behavior ?? true)) {
+        window.AWAdvanceScreens?.(action.advance_screens);
+      }
     }
 
     if (shouldDoBehavior(action.dismiss)) {
@@ -676,8 +864,10 @@ export class WelcomeScreen extends React.PureComponent {
         setScreenMultiSelects={this.props.setScreenMultiSelects}
         activeMultiSelect={this.props.activeMultiSelect}
         setActiveMultiSelect={this.props.setActiveMultiSelect}
-        activeSingleSelect={this.props.activeSingleSelect}
-        setActiveSingleSelect={this.props.setActiveSingleSelect}
+        activeSingleSelectSelections={this.props.activeSingleSelectSelections}
+        setActiveSingleSelectSelection={
+          this.props.setActiveSingleSelectSelection
+        }
         totalNumberOfScreens={this.props.totalNumberOfScreens}
         appAndSystemLocaleInfo={this.props.appAndSystemLocaleInfo}
         negotiatedLanguage={this.props.negotiatedLanguage}
@@ -692,6 +882,13 @@ export class WelcomeScreen extends React.PureComponent {
         forceHideStepsIndicator={this.props.forceHideStepsIndicator}
         ariaRole={this.props.ariaRole}
         aboveButtonStepsIndicator={this.props.aboveButtonStepsIndicator}
+        addonId={this.props.addonId}
+        addonType={this.props.addonType}
+        addonName={this.props.addonName}
+        addonURL={this.props.addonURL}
+        addonIconURL={this.props.addonIconURL}
+        themeScreenshots={this.props.themeScreenshots}
+        isRtamo={this.props.content.isRtamo}
       />
     );
   }
