@@ -13,32 +13,6 @@ const { ExtensionPermissions } = ChromeUtils.importESModule(
 
 loadTestSubscript("head_unified_extensions.js");
 
-const openCustomizationUI = async () => {
-  const customizationReady = BrowserTestUtils.waitForEvent(
-    gNavToolbox,
-    "customizationready"
-  );
-  gCustomizeMode.enter();
-  await customizationReady;
-  ok(
-    CustomizationHandler.isCustomizing(),
-    "expected customizing mode to be enabled"
-  );
-};
-
-const closeCustomizationUI = async () => {
-  const afterCustomization = BrowserTestUtils.waitForEvent(
-    gNavToolbox,
-    "aftercustomization"
-  );
-  gCustomizeMode.exit();
-  await afterCustomization;
-  ok(
-    !CustomizationHandler.isCustomizing(),
-    "expected customizing mode to be disabled"
-  );
-};
-
 add_setup(async function () {
   // Make sure extension buttons added to the navbar will not overflow in the
   // panel, which could happen when a previous test file resizes the current
@@ -49,18 +23,6 @@ add_setup(async function () {
   });
 });
 
-add_task(async function test_button_enabled_by_pref() {
-  const { button } = gUnifiedExtensions;
-  is(button.hidden, false, "expected button to be visible");
-  is(
-    document
-      .getElementById("nav-bar")
-      .getAttribute("unifiedextensionsbuttonshown"),
-    "true",
-    "expected attribute on nav-bar"
-  );
-});
-
 add_task(async function test_open_panel_on_button_click() {
   const extensions = createExtensions([
     { name: "Extension #1" },
@@ -439,112 +401,6 @@ add_task(async function test_list_active_extensions_only() {
   await Promise.all(extensions.map(extension => extension.unload()));
 });
 
-add_task(async function test_button_opens_discopane_when_no_extension() {
-  // The test harness registers regular extensions so we need to mock the
-  // `getActivePolicies` extension to simulate zero extensions installed.
-  const origGetActivePolicies = gUnifiedExtensions.getActivePolicies;
-  gUnifiedExtensions.getActivePolicies = () => [];
-
-  await BrowserTestUtils.withNewTab(
-    { gBrowser, url: "about:robots" },
-    async () => {
-      const { button } = gUnifiedExtensions;
-      ok(button, "expected button");
-
-      // Primary click should open about:addons.
-      const tabPromise = BrowserTestUtils.waitForNewTab(
-        gBrowser,
-        "about:addons",
-        true
-      );
-
-      button.click();
-
-      const tab = await tabPromise;
-      is(
-        gBrowser.currentURI.spec,
-        "about:addons",
-        "expected about:addons to be open"
-      );
-      is(
-        gBrowser.selectedBrowser.contentWindow.gViewController.currentViewId,
-        "addons://discover/",
-        "expected about:addons to show the recommendations"
-      );
-      BrowserTestUtils.removeTab(tab);
-
-      // "Right-click" should open the context menu only.
-      const contextMenu = document.getElementById("toolbar-context-menu");
-      const popupShownPromise = BrowserTestUtils.waitForEvent(
-        contextMenu,
-        "popupshown"
-      );
-      EventUtils.synthesizeMouseAtCenter(button, {
-        type: "contextmenu",
-        button: 2,
-      });
-      await popupShownPromise;
-      await closeChromeContextMenu(contextMenu.id, null);
-    }
-  );
-
-  gUnifiedExtensions.getActivePolicies = origGetActivePolicies;
-});
-
-add_task(
-  async function test_button_opens_extlist_when_no_extension_and_pane_disabled() {
-    // If extensions.getAddons.showPane is set to false, there is no "Recommended" tab,
-    // so we need to make sure we don't navigate to it.
-
-    // The test harness registers regular extensions so we need to mock the
-    // `getActivePolicies` extension to simulate zero extensions installed.
-    const origGetActivePolicies = gUnifiedExtensions.getActivePolicies;
-    gUnifiedExtensions.getActivePolicies = () => [];
-
-    await SpecialPowers.pushPrefEnv({
-      set: [
-        // Set this to another value to make sure not to "accidentally" land on the right page
-        ["extensions.ui.lastCategory", "addons://list/theme"],
-        ["extensions.getAddons.showPane", false],
-      ],
-    });
-
-    await BrowserTestUtils.withNewTab(
-      { gBrowser, url: "about:robots" },
-      async () => {
-        const { button } = gUnifiedExtensions;
-        ok(button, "expected button");
-
-        // Primary click should open about:addons.
-        const tabPromise = BrowserTestUtils.waitForNewTab(
-          gBrowser,
-          "about:addons",
-          true
-        );
-
-        button.click();
-
-        const tab = await tabPromise;
-        is(
-          gBrowser.currentURI.spec,
-          "about:addons",
-          "expected about:addons to be open"
-        );
-        is(
-          gBrowser.selectedBrowser.contentWindow.gViewController.currentViewId,
-          "addons://list/extension",
-          "expected about:addons to show the extension list"
-        );
-        BrowserTestUtils.removeTab(tab);
-      }
-    );
-
-    await SpecialPowers.popPrefEnv();
-
-    gUnifiedExtensions.getActivePolicies = origGetActivePolicies;
-  }
-);
-
 add_task(
   async function test_unified_extensions_panel_not_open_in_customization_mode() {
     const listView = getListView();
