@@ -107,7 +107,10 @@ add_task(async function test_skip_generate_if_non_eng() {
  */
 add_task(async function test_link_preview_with_shift_alt_key_event() {
   await SpecialPowers.pushPrefEnv({
-    set: [["browser.ml.linkPreview.enabled", true]],
+    set: [
+      ["browser.ml.linkPreview.enabled", true],
+      ["browser.ml.linkPreview.shift", true],
+    ],
   });
 
   let stub = sinon.stub(LinkPreview, "_maybeLinkPreview");
@@ -222,6 +225,10 @@ add_task(async function test_link_preview_with_long_press() {
  * Tests that regular typing prevents link preview.
  */
 add_task(async function test_link_preview_with_typing() {
+  await SpecialPowers.pushPrefEnv({
+    set: [["browser.ml.linkPreview.shift", true]],
+  });
+
   const stub = sinon.stub(LinkPreview, "renderLinkPreviewPanel");
 
   XULBrowserWindow.setOverLink(TEST_LINK_URL, {});
@@ -284,6 +291,10 @@ add_task(async function test_link_preview_with_typing() {
  * Tests that certain behaviors do not trigger unexpectedly.
  */
 add_task(async function test_link_preview_no_trigger() {
+  await SpecialPowers.pushPrefEnv({
+    set: [["browser.ml.linkPreview.shift", true]],
+  });
+
   const stub = sinon.stub(LinkPreview, "renderLinkPreviewPanel");
 
   LinkPreview.keyboardComboActive = true;
@@ -407,6 +418,27 @@ add_task(async function test_fetch_page_data() {
   );
 });
 
+/**
+ * Test that Shift-JIS encoding is handled correctly.
+ */
+add_task(async function test_fetch_shift_jis() {
+  await SpecialPowers.pushPrefEnv({
+    set: [["browser.ml.linkPreview.enabled", true]],
+  });
+  const actor =
+    window.browsingContext.currentWindowContext.getActor("LinkPreview");
+  const result = await actor.fetchPageData(
+    "https://example.com/browser/browser/components/genai/tests/browser/data/encodingWithShiftJIS.html"
+  );
+
+  ok(!result.error, "should not have an error");
+  is(
+    result.rawMetaInfo["html:title"],
+    "Shift-JIS テスト",
+    "title should be correct"
+  );
+});
+
 /**
  * Test fetching errors.
  */
