@@ -19,6 +19,13 @@ const FXA_LOGIN_FAILED = 2;
 const SYNC_DISCONNECTED = 0;
 const SYNC_CONNECTED = 1;
 
+const BACKUP_ARCHIVE_ENABLED_PREF_NAME = "browser.backup.archive.enabled";
+const BACKUP_RESTORE_ENABLED_PREF_NAME = "browser.backup.restore.enabled";
+
+ChromeUtils.defineESModuleGetters(lazy, {
+  BackupService: "resource:///modules/backup/BackupService.sys.mjs",
+});
+
 var gSyncPane = {
   get page() {
     return document.getElementById("weavePrefsDeck").selectedIndex;
@@ -37,6 +44,8 @@ var gSyncPane = {
       .getElementById("weavePrefsDeck")
       .removeAttribute("data-hidden-from-search");
 
+    this.updateBackupUIVisibility();
+
     // If the Service hasn't finished initializing, wait for it.
     let xps = Cc["@mozilla.org/weave/service;1"].getService(
       Ci.nsISupports
@@ -47,6 +56,8 @@ var gSyncPane = {
       return;
     }
 
+    this._addPrefObservers();
+
     // it may take some time before all the promises we care about resolve, so
     // pre-load what we can from synchronous sources.
     this._showLoadPage(xps);
@@ -133,7 +144,7 @@ var gSyncPane = {
       if (location.href.includes("action=pair")) {
         gSyncPane.pairAnotherDevice();
       } else if (location.href.includes("action=choose-what-to-sync")) {
-        gSyncPane._chooseWhatToSync(false);
+        gSyncPane._chooseWhatToSync(false, "callToAction");
       }
     }
   },
@@ -239,10 +250,10 @@ var gSyncPane = {
       }
     });
     setEventListener("syncSetup", "command", function () {
-      this._chooseWhatToSync(false);
+      this._chooseWhatToSync(false, "setupSync");
     });
     setEventListener("syncChangeOptions", "command", function () {
-      this._chooseWhatToSync(true);
+      this._chooseWhatToSync(true, "manageSyncSettings");
     });
     setEventListener("syncNow", "command", function () {
       // syncing can take a little time to send the "started" notification, so
@@ -286,11 +297,54 @@ var gSyncPane = {
     }
   },
 
-  async _chooseWhatToSync(isAlreadySyncing) {
+  updateBackupUIVisibility() {
+    let bs = lazy.BackupService.get();
+    let isBackupUIEnabled =
+      bs.archiveEnabledStatus.enabled || bs.restoreEnabledStatus.enabled;
+
+    let dataBackupSectionEl = document.getElementById("dataBackupSection");
+
+    dataBackupSectionEl.toggleAttribute(
+      "data-hidden-from-search",
+      !isBackupUIEnabled
+    );
+
+    let dataBackupGroupEl = document.getElementById("dataBackupGroup");
+    let backupGroupHeaderEl = document.getElementById("backupCategory");
+
+    dataBackupSectionEl.hidden = !isBackupUIEnabled;
+    dataBackupGroupEl.hidden = !isBackupUIEnabled;
+    backupGroupHeaderEl.hidden = !isBackupUIEnabled;
+  },
+
+  _addPrefObservers() {
+    Services.obs.addObserver(
+      this.updateBackupUIVisibility,
+      "backup-service-status-updated"
+    );
+
+    window.addEventListener(
+      "unload",
+      () => {
+        Services.obs.removeObserver(
+          this.updateBackupUIVisibility,
+          "backup-service-status-updated"
+        );
+      },
+      { once: true }
+    );
+  },
+
+  async _chooseWhatToSync(isSyncConfigured, why = null) {
+    // Record the user opening the choose what to sync menu.
+    fxAccounts.telemetry.recordOpenCWTSMenu(why).catch(err => {
+      console.error("Failed to record open CWTS menu event", err);
+    });
+
     // Assuming another device is syncing and we're not,
     // we update the engines selection so the correct
     // checkboxes are pre-filed.
-    if (!isAlreadySyncing) {
+    if (!isSyncConfigured) {
       try {
         await Weave.Service.updateLocalEnginesState();
       } catch (err) {
@@ -298,7 +352,7 @@ var gSyncPane = {
       }
     }
     let params = {};
-    if (isAlreadySyncing) {
+    if (isSyncConfigured) {
       // If we are already syncing then we also offer to disconnect.
       params.disconnectFun = () => this.disconnectSync();
     }
@@ -306,18 +360,28 @@ var gSyncPane = {
       "chrome://browser/content/preferences/dialogs/syncChooseWhatToSync.xhtml",
       {
         closingCallback: event => {
-          if (!isAlreadySyncing && event.detail.button == "accept") {
-            // We weren't syncing but the user has accepted the dialog - so we
-            // want to start!
-            fxAccounts.telemetry
-              .recordConnection(["sync"], "ui")
-              .then(() => {
-                this.updateSyncUI();
-                return Weave.Service.configure();
-              })
-              .catch(err => {
-                console.error("Failed to enable sync", err);
+          if (event.detail.button == "accept") {
+            // Sync wasn't previously configured, but the user has accepted
+            // so we want to now start syncing!
+            if (!isSyncConfigured) {
+              fxAccounts.telemetry
+                .recordConnection(["sync"], "ui")
+                .then(() => {
+                  this.updateSyncUI();
+                  return Weave.Service.configure();
+                })
+                .catch(err => {
+                  console.error("Failed to enable sync", err);
+                });
+            } else {
+              // User is already configured and have possibly changed the engines they want to
+              // sync, so we should let the server know immediately
+              // if the user is currently syncing, we queue another sync after
+              // to ensure we caught their updates
+              Services.tm.dispatchToMainThread(() => {
+                Weave.Service.queueSync("cwts");
               });
+            }
           }
           // When the modal closes we want to remove any query params
           // so it doesn't open on subsequent visits (and will reload)
@@ -439,10 +503,10 @@ var gSyncPane = {
   },
 
   _getEntryPoint() {
-    let params = new URLSearchParams(
-      document.URL.split("#")[0].split("?")[1] || ""
-    );
-    return params.get("entrypoint") || "preferences";
+    let params = URL.fromURI(document.documentURIObject).searchParams;
+    let entryPoint = params.get("entrypoint") || "preferences";
+    entryPoint = entryPoint.replace(/[^-.\w]/g, "");
+    return entryPoint;
   },
 
   openContentInBrowser(url, options) {
