@@ -14,12 +14,13 @@ const { DeferredTask } = ChromeUtils.importESModule(
 
 const toolsNameMap = {
   viewGenaiChatSidebar: "aichat",
+  viewGenaiPageAssistSidebar: "aipageassist",
+  viewGenaiSmartAssistSidebar: "aismartassist",
   viewTabsSidebar: "syncedtabs",
   viewHistorySidebar: "history",
   viewBookmarksSidebar: "bookmarks",
   viewCPMSidebar: "passwords",
 };
-const EXPAND_ON_HOVER_DEBOUNCE_RATE_MS = 200;
 const EXPAND_ON_HOVER_DEBOUNCE_TIMEOUT_MS = 1000;
 const LAUNCHER_SPLITTER_WIDTH = 4;
 
@@ -186,6 +187,34 @@ var SidebarController = {
       }
     );
 
+    this.registerPrefSidebar(
+      "browser.ml.pageAssist.enabled",
+      "viewGenaiPageAssistSidebar",
+      {
+        name: "aipageassist",
+        elementId: "sidebar-switcher-genai-page-assist",
+        url: "chrome://browser/content/genai/pageAssist.html",
+        menuId: "menu_genaiPageAssistSidebar",
+        menuL10nId: "menu-view-genai-page-assist",
+        revampL10nId: "sidebar-menu-genai-page-assist-label",
+        iconUrl: "chrome://browser/skin/reader-mode.svg",
+      }
+    );
+
+    this.registerPrefSidebar(
+      "browser.ml.smartAssist.enabled",
+      "viewGenaiSmartAssistSidebar",
+      {
+        name: "aismartassist",
+        elementId: "sidebar-switcher-genai-smart-assist",
+        url: "chrome://browser/content/genai/smartAssist.html",
+        menuId: "menu_genaiSmartAssistSidebar",
+        menuL10nId: "menu-view-genai-smart-assist",
+        revampL10nId: "sidebar-menu-genai-smart-assist-label",
+        iconUrl: "chrome://browser/skin/trending.svg",
+      }
+    );
+
     this.registerPrefSidebar(
       "browser.contextual-password-manager.enabled",
       "viewCPMSidebar",
@@ -418,7 +447,10 @@ var SidebarController = {
         );
       }
       this.revampComponentsLoaded = true;
-      this._state.initializeState();
+      this._state.initializeState(this._showLauncherAfterInit);
+      // clear the flag after we've used it
+      delete this._showLauncherAfterInit;
+
       document.getElementById("sidebar-header").hidden = true;
       if (!this._mainResizeObserverAdded) {
         this._mainResizeObserver.observe(this.sidebarMain);
@@ -543,6 +575,15 @@ var SidebarController = {
     this._disablePinnedTabsDragging();
   },
 
+  /**
+   * Keep track when sidebar.revamp is enabled by the user via about:preferences UI
+   *
+   * @param {boolean} isEnabled
+   */
+  enabledViaSettings(isEnabled = false) {
+    this._showLauncherAfterInit = isEnabled;
+  },
+
   /**
    * Handle the launcher being resized (either manually or programmatically).
    *
@@ -815,14 +856,6 @@ var SidebarController = {
     if (!this.sidebarRevampEnabled) {
       this._state.launcherVisible = false;
       document.getElementById("sidebar-header").hidden = false;
-      // Disable vertical tabs if revamped sidebar is turned off
-      if (this.sidebarVerticalTabsEnabled) {
-        Services.prefs.setBoolPref("sidebar.verticalTabs", false);
-      }
-    } else {
-      // initial launcher visibleness with sidebar.revamp is is one of the
-      // default properties managed by SidebarState
-      this._state.launcherVisible = this._state.defaultLauncherVisible;
     }
     if (!this._sidebars.get(this.lastOpenedId)) {
       this.lastOpenedId = this.DEFAULT_SIDEBAR_ID;
@@ -1355,7 +1388,15 @@ var SidebarController = {
         this.dismissSidebarBadge(commandID);
       }
 
-      if (this.sidebarRevampEnabled && badgePref && isSidebarClosed) {
+      // Show badge on toolbar if we would have shown it on a visible tool but sidebar is closed
+      const tool = this.toolsAndExtensions.get(commandID);
+      if (
+        this.sidebarRevampEnabled &&
+        badgePref &&
+        !tool.disabled &&
+        !tool.hidden &&
+        isSidebarClosed
+      ) {
         this._showToolbarButtonBadge();
       } else {
         this._clearToolbarButtonBadge();
@@ -1372,13 +1413,7 @@ var SidebarController = {
 
   async _removeHoverStateBlocker() {
     if (this._hoverBlockerCount == 1) {
-      // Manually check mouse position
-      let isHovered;
-      MousePosTracker._callListener({
-        onMouseEnter: () => (isHovered = true),
-        onMouseLeave: () => (isHovered = false),
-        getMouseTargetRect: () => this.getMouseTargetRect(),
-      });
+      let isHovered = this._checkIsHoveredOverLauncher();
 
       // Collapse sidebar if needed
       if (this._state.launcherExpanded && !isHovered) {
@@ -1392,7 +1427,9 @@ var SidebarController = {
       // Re-add MousePosTracker listener
       MousePosTracker.addListener(this);
     }
-    this._hoverBlockerCount--;
+    if (this._hoverBlockerCount > 0) {
+      this._hoverBlockerCount--;
+    }
   },
 
   _showToolbarButtonBadge() {
@@ -2062,6 +2099,20 @@ var SidebarController = {
     }
   },
 
+  /**
+   * Use MousePosTracker to manually check for hover state over launcher
+   */
+  _checkIsHoveredOverLauncher() {
+    // Manually check mouse position
+    let isHovered;
+    MousePosTracker._callListener({
+      onMouseEnter: () => (isHovered = true),
+      onMouseLeave: () => (isHovered = false),
+      getMouseTargetRect: () => this.getMouseTargetRect(),
+    });
+    return isHovered;
+  },
+
   /**
    * Record to Glean when any of the sidebar icons are clicked.
    *
@@ -2162,6 +2213,7 @@ var SidebarController = {
       this._animateSidebarMain();
     }
     this._state.launcherExpanded = true;
+    this._mouseEnterDeferred.resolve();
   },
 
   onMouseLeave() {
@@ -2169,6 +2221,7 @@ var SidebarController = {
       return;
     }
     this.mouseEnterTask.disarm();
+    this._mouseEnterDeferred.resolve();
     const contentArea = document.getElementById("tabbrowser-tabbox");
     this._box.toggleAttribute("sidebar-launcher-hovered", false);
     contentArea.toggleAttribute("sidebar-launcher-hovered", false);
@@ -2183,16 +2236,26 @@ var SidebarController = {
     if (this._state.launcherExpanded) {
       return;
     }
+    this._mouseEnterDeferred = Promise.withResolvers();
     this.mouseEnterTask = new DeferredTask(
       () => {
-        this.debouncedMouseEnter();
+        let isHovered = this._checkIsHoveredOverLauncher();
+
+        // Only expand sidebar if mouse is still hovering over sidebar launcher
+        if (isHovered) {
+          this.debouncedMouseEnter();
+        }
       },
-      EXPAND_ON_HOVER_DEBOUNCE_RATE_MS,
+      this._animationExpandOnHoverDelayDurationMs,
       EXPAND_ON_HOVER_DEBOUNCE_TIMEOUT_MS
     );
     this.mouseEnterTask?.arm();
   },
 
+  get expandOnHoverComplete() {
+    return this._mouseEnterDeferred?.promise || Promise.resolve();
+  },
+
   async setLauncherCollapsedWidth() {
     let browserEl = document.getElementById("browser");
     if (this.getUIState().launcherExpanded) {
@@ -2231,12 +2294,18 @@ var SidebarController = {
     switch (e.type) {
       case "popupshown":
         /* Temporarily remove MousePosTracker listener when a context menu is open */
-        if (e.composedTarget.id !== "tab-preview-panel") {
+        if (
+          e.composedTarget.id !== "tab-preview-panel" &&
+          e.composedTarget.tagName !== "tooltip"
+        ) {
           this._addHoverStateBlocker();
         }
         break;
       case "popuphidden":
-        if (e.composedTarget.id !== "tab-preview-panel") {
+        if (
+          e.composedTarget.id !== "tab-preview-panel" &&
+          e.composedTarget.tagName !== "tooltip"
+        ) {
           await this._removeHoverStateBlocker();
         }
         break;
@@ -2262,10 +2331,12 @@ var SidebarController = {
       document.addEventListener("popupshown", this);
       document.addEventListener("popuphidden", this);
       // Reset user-preferred height
-      this.sidebarMain.buttonGroup.style.height = this._state.launcherExpanded
+      this.sidebarMain.buttonsWrapper.style.height = this._state
+        .launcherExpanded
         ? ""
         : "0";
     } else {
+      this._removeHoverStateBlocker();
       MousePosTracker.removeListener(this);
       if (!this.mouseOverTask?.isFinalized) {
         this.mouseOverTask?.finalize();
@@ -2371,6 +2442,12 @@ XPCOMUtils.defineLazyPreferenceGetter(
   "sidebar.animation.expand-on-hover.duration-ms",
   400
 );
+XPCOMUtils.defineLazyPreferenceGetter(
+  SidebarController,
+  "_animationExpandOnHoverDelayDurationMs",
+  "sidebar.animation.expand-on-hover.delay-duration-ms",
+  200
+);
 XPCOMUtils.defineLazyPreferenceGetter(
   SidebarController,
   "sidebarRevampEnabled",
@@ -2474,18 +2551,3 @@ XPCOMUtils.defineLazyPreferenceGetter(
     }
   }
 );
-
-XPCOMUtils.defineLazyPreferenceGetter(
-  SidebarController,
-  "revampDefaultLauncherVisible",
-  "sidebar.revamp.defaultLauncherVisible",
-  false,
-  (_aPreference, _previousValue, _newValue) => {
-    if (
-      !SidebarController.uninitializing &&
-      !SidebarController.inSingleTabWindow
-    ) {
-      SidebarController._state.updateVisibility();
-    }
-  }
-);
