@@ -72,17 +72,13 @@
        */
       this.muteReason = undefined;
 
-      this.mOverCloseButton = false;
-
-      this.mCorrespondingMenuitem = null;
-
       this.closing = false;
     }
 
     static get inheritedAttributes() {
       return {
         ".tab-background":
-          "selected=visuallyselected,fadein,multiselected,dragover-createGroup",
+          "selected=visuallyselected,fadein,multiselected,dragover-groupTarget",
         ".tab-group-line": "selected=visuallyselected,multiselected",
         ".tab-loading-burst": "pinned,bursting,notselectedsinceload",
         ".tab-content":
@@ -94,7 +90,7 @@
         ".tab-icon-pending":
           "fadein,pinned,busy,progress,selected=visuallyselected,pendingicon",
         ".tab-icon-image":
-          "src=image,triggeringprincipal=iconloadingprincipal,requestcontextid,fadein,pinned,selected=visuallyselected,busy,crashed,sharing,pictureinpicture,pending,discarded",
+          "src=image,requestcontextid,fadein,pinned,selected=visuallyselected,busy,crashed,sharing,pictureinpicture,pending,discarded",
         ".tab-sharing-icon-overlay": "sharing,selected=visuallyselected,pinned",
         ".tab-icon-overlay":
           "sharing,pictureinpicture,crashed,busy,soundplaying,soundplaying-scheduledremoval,pinned,muted,blocked,selected=visuallyselected,activemedia-blocked",
@@ -113,6 +109,7 @@
     #lastGroup;
     connectedCallback() {
       this.#updateOnTabGrouped();
+      this.#updateOnTabSplit();
       this.#lastGroup = this.group;
 
       this.initialize();
@@ -120,6 +117,7 @@
 
     disconnectedCallback() {
       this.#updateOnTabUngrouped();
+      this.#updateOnTabUnsplit();
     }
 
     initialize() {
@@ -225,7 +223,9 @@
 
     get visible() {
       return (
-        this.isOpen && !this.hidden && (!this.group?.collapsed || this.selected)
+        this.isOpen &&
+        !this.hidden &&
+        (!this.group || this.group.isTabVisibleInGroup(this))
       );
     }
 
@@ -380,6 +380,15 @@
       return null;
     }
 
+    get splitview() {
+      if (
+        this.parentElement?.parentElement?.tagName == "tab-split-view-wrapper"
+      ) {
+        return this.parentElement.parentElement;
+      }
+      return null;
+    }
+
     updateLastAccessed(aDate) {
       this._lastAccessed = this.selected ? Infinity : aDate || Date.now();
     }
@@ -407,15 +416,11 @@
     }
 
     on_mouseover(event) {
-      if (event.target.classList.contains("tab-close-button")) {
-        this.mOverCloseButton = true;
-      }
-
       if (!this.visible) {
         return;
       }
 
-      let tabToWarm = this.mOverCloseButton
+      let tabToWarm = event.target.classList.contains("tab-close-button")
         ? gBrowser._findTabToBlurTo(this)
         : this;
       gBrowser.warmupTab(tabToWarm);
@@ -427,10 +432,6 @@
     }
 
     on_mouseout(event) {
-      if (event.target.classList.contains("tab-close-button")) {
-        this.mOverCloseButton = false;
-      }
-
       // If the new target is not part of this tab then this is a mouseleave event.
       if (!this.contains(event.relatedTarget)) {
         this._mouseleave();
@@ -446,7 +447,7 @@
       if (event.eventPhase == Event.CAPTURING_PHASE) {
         this.style.MozUserFocus = "";
       } else if (
-        this.mOverCloseButton ||
+        event.target.classList?.contains("tab-close-button") ||
         gSharedTabWarning.willShowSharedTabWarning(this)
       ) {
         event.stopPropagation();
@@ -767,6 +768,33 @@
         this.removeAttribute("aria-setsize");
       }
     }
+
+    #updateOnTabSplit() {
+      if (this.splitview) {
+        this.setAttribute("aria-level", 2);
+
+        // Add "Split view" to label if tab is within a split view
+        let splitViewLabel = gBrowser.tabLocalization.formatValueSync(
+          "tabbrowser-tab-label-tab-split-view"
+        );
+        this.setAttribute(
+          "aria-label",
+          `${this.getAttribute("label")}, ${splitViewLabel}`
+        );
+      }
+    }
+
+    #updateOnTabUnsplit() {
+      if (this.splitview) {
+        this.setAttribute("aria-level", 1);
+        // `posinset` and `setsize` only need to be set explicitly
+        // on split view tabs so that a11y tools can tell users that a
+        // given tab is "1 of 2" in the split view, for example.
+        this.removeAttribute("aria-posinset");
+        this.removeAttribute("aria-setsize");
+        this.removeAttribute("aria-label");
+      }
+    }
   }
 
   customElements.define("tabbrowser-tab", MozTabbrowserTab, {
