@@ -11,7 +11,10 @@
 ChromeUtils.defineESModuleGetters(this, {
   AmpMatchingStrategy:
     "moz-src:///toolkit/components/uniffi-bindgen-gecko-js/components/generated/RustSuggest.sys.mjs",
-  AmpSuggestions: "resource:///modules/urlbar/private/AmpSuggestions.sys.mjs",
+  AmpSuggestions:
+    "moz-src:///browser/components/urlbar/private/AmpSuggestions.sys.mjs",
+  SuggestBackendRust:
+    "moz-src:///browser/components/urlbar/private/SuggestBackendRust.sys.mjs",
   SuggestionProvider:
     "moz-src:///toolkit/components/uniffi-bindgen-gecko-js/components/generated/RustSuggest.sys.mjs",
 });
@@ -55,6 +58,7 @@ const REMOTE_SETTINGS_RESULTS = [
     advertiser: "HttpAdvertiser",
     iab_category: "22 - Shopping",
     icon: "1234",
+    serp_categories: [2],
   },
   {
     id: 4,
@@ -167,6 +171,8 @@ function expectedHttpResult() {
     clickUrl: suggestion.click_url,
     blockId: suggestion.id,
     advertiser: suggestion.advertiser,
+    categories: suggestion.serp_categories,
+    suggestedIndex: -1,
   });
 }
 
@@ -181,6 +187,7 @@ function expectedHttpsResult() {
     clickUrl: suggestion.click_url,
     blockId: suggestion.id,
     advertiser: suggestion.advertiser,
+    suggestedIndex: -1,
   });
 }
 
@@ -207,7 +214,6 @@ add_setup(async function init() {
     Ci.nsISearchService.CHANGE_REASON_UNKNOWN
   );
 
-  UrlbarPrefs.set("scotchBonnet.enableOverride", false);
   UrlbarPrefs.set("quicksuggest.ampTopPickCharThreshold", 0);
 
   await QuickSuggestTestUtils.ensureQuickSuggestInit();
@@ -311,7 +317,7 @@ add_task(async function sponsoredOnly_sponsored() {
   });
   await check_results({
     context,
-    matches: [QuickSuggestTestUtils.ampResult()],
+    matches: [QuickSuggestTestUtils.ampResult({ suggestedIndex: -1 })],
   });
 
   // The title should include the full keyword and em dash, and the part of the
@@ -356,7 +362,7 @@ add_task(async function both_sponsored() {
   });
   await check_results({
     context,
-    matches: [QuickSuggestTestUtils.ampResult()],
+    matches: [QuickSuggestTestUtils.ampResult({ suggestedIndex: -1 })],
   });
 });
 
@@ -429,7 +435,7 @@ add_task(async function caseInsensitiveAndLeadingSpaces() {
   });
   await check_results({
     context,
-    matches: [QuickSuggestTestUtils.ampResult()],
+    matches: [QuickSuggestTestUtils.ampResult({ suggestedIndex: -1 })],
   });
 });
 
@@ -454,7 +460,9 @@ add_task(async function emptySearchStringsAndSpaces() {
       matches: [],
     });
     Assert.ok(
-      !(await UrlbarProviderQuickSuggest.isActive(context)),
+      !(await UrlbarProvidersManager.getProvider(
+        UrlbarProviderQuickSuggest.name
+      ).isActive(context)),
       "Provider should not be active for search string: " + msg
     );
   }
@@ -750,6 +758,7 @@ add_task(async function maxResults() {
         keyword: "maxresults",
         title: "maxresults 0",
         url: "https://example.com/maxresults/0",
+        suggestedIndex: -1,
       }),
     ],
   });
@@ -785,6 +794,7 @@ add_task(async function manySuggestResults_visible() {
         keyword: "maxresults",
         title: "maxresults 0",
         url: "https://example.com/maxresults/0",
+        suggestedIndex: -1,
       }),
     ],
     expectedOtherResultsCount: UrlbarPrefs.get("maxRichResults") - 1,
@@ -810,6 +820,7 @@ add_task(async function manySuggestResults_hiddenExposures() {
         keyword: "maxresults",
         title: "maxresults " + index,
         url: "https://example.com/maxresults/" + index,
+        suggestedIndex: -1,
       }),
       exposureTelemetry: UrlbarUtils.EXPOSURE_TELEMETRY.HIDDEN,
     });
@@ -850,11 +861,11 @@ async function doManySuggestResultsTest({
   let maxResults = UrlbarPrefs.get("maxRichResults");
   for (let i = 0; i < maxResults; i++) {
     otherResults.push(
-      new UrlbarResult(
-        UrlbarUtils.RESULT_TYPE.URL,
-        UrlbarUtils.RESULT_SOURCE.HISTORY,
-        { url: "http://example.com/history/" + i }
-      )
+      new UrlbarResult({
+        type: UrlbarUtils.RESULT_TYPE.URL,
+        source: UrlbarUtils.RESULT_SOURCE.HISTORY,
+        payload: { url: "http://example.com/history/" + i },
+      })
     );
   }
 
@@ -870,8 +881,8 @@ async function doManySuggestResultsTest({
       isPrivate: false,
     }),
     matches: [
-      ...expectedSuggestResults,
       ...otherResults.slice(0, expectedOtherResultsCount),
+      ...expectedSuggestResults,
     ],
   });
 
@@ -1214,7 +1225,10 @@ add_task(async function dismissResult() {
 
   let tests = [
     // [suggestion, expected result]
-    [REMOTE_SETTINGS_RESULTS[0], QuickSuggestTestUtils.ampResult()],
+    [
+      REMOTE_SETTINGS_RESULTS[0],
+      QuickSuggestTestUtils.ampResult({ suggestedIndex: -1 }),
+    ],
     [REMOTE_SETTINGS_RESULTS[1], QuickSuggestTestUtils.wikipediaResult()],
     [REMOTE_SETTINGS_RESULTS[2], expectedHttpResult()],
     [REMOTE_SETTINGS_RESULTS[3], expectedHttpsResult()],
@@ -1393,6 +1407,7 @@ add_task(async function tabToSearch() {
   // have the same behavior.
   UrlbarPrefs.set("suggest.quicksuggest.sponsored", true);
   await QuickSuggestTestUtils.forceSync();
+  UrlbarPrefs.set("suggest.quickactions", false);
 
   Services.prefs.setBoolPref(
     "browser.urlbar.quicksuggest.sponsoredPriority",
@@ -1442,7 +1457,7 @@ add_task(async function tabToSearch() {
         ),
         providesSearchMode: true,
         query: "",
-        providerName: "TabToSearch",
+        providerName: "UrlbarProviderTabToSearch",
         satisfiesAutofillThreshold: true,
       }),
       // Suggest best match
@@ -1459,6 +1474,90 @@ add_task(async function tabToSearch() {
   await extension.unload();
 
   UrlbarPrefs.clear("tabToSearch.onboard.interactionsLeft");
+  UrlbarPrefs.clear("suggest.quickactions");
+  Services.prefs.clearUserPref("browser.search.suggest.enabled");
+  Services.prefs.clearUserPref("browser.urlbar.quicksuggest.sponsoredPriority");
+});
+
+// When a Suggest best match and a global action are shown in the same
+// search, both will have a `suggestedIndex` value of 1. The global action should
+// appear first.
+add_task(async function globalAction() {
+  // We'll use a sponsored priority result as the best match result. Different
+  // types of Suggest results can appear as best matches, and they all should
+  // have the same behavior.
+  UrlbarPrefs.set("suggest.quicksuggest.sponsored", true);
+  await QuickSuggestTestUtils.forceSync();
+
+  Services.prefs.setBoolPref(
+    "browser.urlbar.quicksuggest.sponsoredPriority",
+    true
+  );
+
+  // Disable search suggestions so we don't need to expect them below.
+  Services.prefs.setBoolPref("browser.search.suggest.enabled", false);
+
+  // Set prefs to prevent quick actions onboarding label from showing.
+  UrlbarPrefs.set("quickactions.timesToShowOnboardingLabel", 3);
+  UrlbarPrefs.set("quickactions.timesShownOnboardingLabel", 3);
+
+  let engineURL = `https://example.com/`;
+  let extension = await SearchTestUtils.installSearchExtension(
+    {
+      name: "Amp",
+      search_url: engineURL,
+    },
+    { skipUnload: true }
+  );
+  let engine = Services.search.getEngineByName("Amp");
+
+  await PlacesTestUtils.addVisits(engineURL);
+
+  let context = createContext(SPONSORED_SEARCH_STRING, {
+    isPrivate: false,
+  });
+
+  await check_results({
+    context,
+    matches: [
+      // search heuristic
+      makeSearchResult(context, {
+        engineName: Services.search.defaultEngine.name,
+        engineIconUri: await Services.search.defaultEngine.getIconURL(),
+        heuristic: true,
+      }),
+
+      // "Search with engine" global action.
+      makeGlobalActionsResult({
+        actionsResults: [
+          {
+            providerName: "ActionsProviderContextualSearch",
+          },
+        ],
+        providesSearchMode: true,
+        engine: engine.name,
+        query: "",
+        input: "",
+        inputLength: context.searchString.length,
+        showOnboardingLabel: false,
+      }),
+
+      // Suggest best match
+      expectedSponsoredPriorityResult(),
+
+      // visit
+      makeVisitResult(context, {
+        uri: engineURL,
+        title: `test visit for ${engineURL}`,
+      }),
+    ],
+  });
+
+  await cleanupPlaces();
+  await extension.unload();
+
+  UrlbarPrefs.clear("quickactions.timesToShowOnboardingLabel");
+  UrlbarPrefs.clear("quickactions.timesShownOnboardingLabel");
   Services.prefs.clearUserPref("browser.search.suggest.enabled");
   Services.prefs.clearUserPref("browser.urlbar.quicksuggest.sponsoredPriority");
 });
@@ -1531,6 +1630,7 @@ add_task(async function keywordLengthThreshold() {
               title: "Suggestion with 1-char keyword",
               url: "http://example.com/1-char-keyword",
               originalUrl: "http://example.com/1-char-keyword",
+              suggestedIndex: -1,
             }),
           ],
     });
@@ -1636,6 +1736,7 @@ add_task(async function ampTopPickCharThreshold() {
         });
       } else {
         expectedResult = QuickSuggestTestUtils.ampResult({
+          suggestedIndex: -1,
           keyword,
           fullKeyword,
           title: "AMP suggestion with full keyword and prefix keywords",
@@ -1698,6 +1799,7 @@ add_task(async function ampTopPickCharThreshold_zero() {
         fullKeyword,
         title: "AMP suggestion with full keyword and prefix keywords",
         url: "https://example.com/amp-full-keyword",
+        suggestedIndex: -1,
       });
     }
 
@@ -1852,7 +1954,7 @@ add_task(async function offline_amp_disabled() {
         providers: [UrlbarProviderQuickSuggest.name],
         isPrivate: false,
       }),
-      matches: [QuickSuggestTestUtils.ampResult()],
+      matches: [QuickSuggestTestUtils.ampResult({ suggestedIndex: -1 })],
     });
 
     // Now disable the pref and try again.
@@ -1938,6 +2040,7 @@ add_task(async function online_amp_disabled() {
             icon: "https://example.com/amp-icon",
             iabCategory: "22 - Shopping",
             requestId: "request_id",
+            suggestedIndex: -1,
           }),
         ],
       });
@@ -2021,6 +2124,265 @@ async function doMerinoTest(callback) {
   UrlbarPrefs.clear("quicksuggest.dataCollection.enabled");
 }
 
+add_task(async function mergeRustProviderConstraints() {
+  let tests = [
+    {
+      a: null,
+      b: null,
+      expected: null,
+    },
+
+    // b is null
+    {
+      a: {},
+      b: null,
+      expected: {},
+    },
+    {
+      a: { ampAlternativeMatching: 1 },
+      b: null,
+      expected: { ampAlternativeMatching: 1 },
+    },
+    {
+      a: { dynamicSuggestionTypes: [] },
+      b: null,
+      expected: { dynamicSuggestionTypes: [] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa"] },
+      b: null,
+      expected: { dynamicSuggestionTypes: ["aaa"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+      b: null,
+      expected: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"], ampAlternativeMatching: 1 },
+      b: null,
+      expected: {
+        dynamicSuggestionTypes: ["aaa", "bbb"],
+        ampAlternativeMatching: 1,
+      },
+    },
+
+    // b is an empty object
+    {
+      a: {},
+      b: {},
+      expected: {},
+    },
+    {
+      a: { ampAlternativeMatching: 1 },
+      b: {},
+      expected: { ampAlternativeMatching: 1 },
+    },
+    {
+      a: { dynamicSuggestionTypes: [] },
+      b: {},
+      expected: { dynamicSuggestionTypes: [] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa"] },
+      b: {},
+      expected: { dynamicSuggestionTypes: ["aaa"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+      b: {},
+      expected: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"], ampAlternativeMatching: 1 },
+      b: {},
+      expected: {
+        dynamicSuggestionTypes: ["aaa", "bbb"],
+        ampAlternativeMatching: 1,
+      },
+    },
+
+    // b is { ampAlternativeMatching: 1 }
+    {
+      a: {},
+      b: { ampAlternativeMatching: 1 },
+      expected: { ampAlternativeMatching: 1 },
+    },
+    {
+      a: { ampAlternativeMatching: 1 },
+      b: { ampAlternativeMatching: 1 },
+      expected: { ampAlternativeMatching: 1 },
+    },
+    {
+      a: { dynamicSuggestionTypes: [] },
+      b: { ampAlternativeMatching: 1 },
+      expected: { dynamicSuggestionTypes: [], ampAlternativeMatching: 1 },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa"] },
+      b: { ampAlternativeMatching: 1 },
+      expected: { dynamicSuggestionTypes: ["aaa"], ampAlternativeMatching: 1 },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+      b: { ampAlternativeMatching: 1 },
+      expected: {
+        dynamicSuggestionTypes: ["aaa", "bbb"],
+        ampAlternativeMatching: 1,
+      },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"], ampAlternativeMatching: 1 },
+      b: { ampAlternativeMatching: 1 },
+      expected: {
+        dynamicSuggestionTypes: ["aaa", "bbb"],
+        ampAlternativeMatching: 1,
+      },
+    },
+
+    // b is { dynamicSuggestionTypes: [] }
+    {
+      a: {},
+      b: { dynamicSuggestionTypes: [] },
+      expected: { dynamicSuggestionTypes: [] },
+    },
+    {
+      a: { ampAlternativeMatching: 1 },
+      b: { dynamicSuggestionTypes: [] },
+      expected: { dynamicSuggestionTypes: [], ampAlternativeMatching: 1 },
+    },
+    {
+      a: { dynamicSuggestionTypes: [] },
+      b: { dynamicSuggestionTypes: [] },
+      expected: { dynamicSuggestionTypes: [] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa"] },
+      b: { dynamicSuggestionTypes: [] },
+      expected: { dynamicSuggestionTypes: ["aaa"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+      b: { dynamicSuggestionTypes: [] },
+      expected: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"], ampAlternativeMatching: 1 },
+      b: { dynamicSuggestionTypes: [] },
+      expected: {
+        dynamicSuggestionTypes: ["aaa", "bbb"],
+        ampAlternativeMatching: 1,
+      },
+    },
+
+    // b is { dynamicSuggestionTypes: ["bbb"] }
+    {
+      a: {},
+      b: { dynamicSuggestionTypes: ["bbb"] },
+      expected: { dynamicSuggestionTypes: ["bbb"] },
+    },
+    {
+      a: { ampAlternativeMatching: 1 },
+      b: { dynamicSuggestionTypes: ["bbb"] },
+      expected: { dynamicSuggestionTypes: ["bbb"], ampAlternativeMatching: 1 },
+    },
+    {
+      a: { dynamicSuggestionTypes: [] },
+      b: { dynamicSuggestionTypes: ["bbb"] },
+      expected: { dynamicSuggestionTypes: ["bbb"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa"] },
+      b: { dynamicSuggestionTypes: ["bbb"] },
+      expected: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["bbb"] },
+      b: { dynamicSuggestionTypes: ["bbb"] },
+      expected: { dynamicSuggestionTypes: ["bbb"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+      b: { dynamicSuggestionTypes: ["bbb"] },
+      expected: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"], ampAlternativeMatching: 1 },
+      b: { dynamicSuggestionTypes: ["bbb"] },
+      expected: {
+        dynamicSuggestionTypes: ["aaa", "bbb"],
+        ampAlternativeMatching: 1,
+      },
+    },
+
+    // b is { dynamicSuggestionTypes: ["bbb", "ddd"] }
+    {
+      a: {},
+      b: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+      expected: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+    },
+    {
+      a: { ampAlternativeMatching: 1 },
+      b: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+      expected: {
+        dynamicSuggestionTypes: ["bbb", "ddd"],
+        ampAlternativeMatching: 1,
+      },
+    },
+    {
+      a: { dynamicSuggestionTypes: [] },
+      b: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+      expected: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa"] },
+      b: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+      expected: { dynamicSuggestionTypes: ["aaa", "bbb", "ddd"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["bbb"] },
+      b: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+      expected: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb"] },
+      b: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+      expected: { dynamicSuggestionTypes: ["aaa", "bbb", "ddd"] },
+    },
+    {
+      a: { dynamicSuggestionTypes: ["aaa", "bbb", "ccc"] },
+      b: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+      expected: { dynamicSuggestionTypes: ["aaa", "bbb", "ccc", "ddd"] },
+    },
+    {
+      a: {
+        dynamicSuggestionTypes: ["aaa", "bbb", "ccc"],
+        ampAlternativeMatching: 1,
+      },
+      b: { dynamicSuggestionTypes: ["bbb", "ddd"] },
+      expected: {
+        dynamicSuggestionTypes: ["aaa", "bbb", "ccc", "ddd"],
+        ampAlternativeMatching: 1,
+      },
+    },
+  ];
+
+  for (let { a, b, expected } of tests) {
+    for (let [first, second] of [
+      [a, b],
+      [b, a],
+    ]) {
+      info("Doing test: " + JSON.stringify({ first, second }));
+      let actual = SuggestBackendRust.mergeProviderConstraints(first, second);
+      Assert.deepEqual(
+        actual,
+        expected,
+        "Expected merged constraints with " + JSON.stringify({ first, second })
+      );
+    }
+  }
+});
+
 async function resetRemoteSettingsData(data = REMOTE_SETTINGS_RESULTS) {
   let isAmp = suggestion => suggestion.iab_category == "22 - Shopping";
   await QuickSuggestTestUtils.setRemoteSettingsRecords([
