@@ -7,7 +7,10 @@ import { DSEmptyState } from "../DSEmptyState/DSEmptyState";
 import { DSCard, PlaceholderDSCard } from "../DSCard/DSCard";
 import { useSelector } from "react-redux";
 import { actionCreators as ac, actionTypes as at } from "common/Actions.mjs";
-import { useIntersectionObserver } from "../../../lib/utils";
+import {
+  selectWeatherPlacement,
+  useIntersectionObserver,
+} from "../../../lib/utils";
 import { SectionContextMenu } from "../SectionContextMenu/SectionContextMenu";
 import { InterestPicker } from "../InterestPicker/InterestPicker";
 import { AdBanner } from "../AdBanner/AdBanner.jsx";
@@ -15,6 +18,7 @@ import { PersonalizedCard } from "../PersonalizedCard/PersonalizedCard";
 import { FollowSectionButtonHighlight } from "../FeatureHighlight/FollowSectionButtonHighlight";
 import { MessageWrapper } from "content-src/components/MessageWrapper/MessageWrapper";
 import { TrendingSearches } from "../TrendingSearches/TrendingSearches.jsx";
+import { Weather } from "../../Weather/Weather.jsx";
 
 // Prefs
 const PREF_SECTIONS_CARDS_ENABLED = "discoverystream.sections.cards.enabled";
@@ -41,6 +45,9 @@ const PREF_TRENDING_SEARCH = "trendingSearch.enabled";
 const PREF_TRENDING_SEARCH_SYSTEM = "system.trendingSearch.enabled";
 const PREF_SEARCH_ENGINE = "trendingSearch.defaultSearchEngine";
 const PREF_TRENDING_SEARCH_VARIANT = "trendingSearch.variant";
+const PREF_DAILY_BRIEF_SECTIONID = "discoverystream.dailyBrief.sectionId";
+const PREF_SPOCS_STARTUPCACHE_ENABLED =
+  "discoverystream.spocs.startupCache.enabled";
 
 function getLayoutData(
   responsiveLayouts,
@@ -139,10 +146,11 @@ function CardSection({
   dispatch,
   type,
   firstVisibleTimestamp,
-  spocMessageVariant,
   ctaButtonVariant,
   ctaButtonSponsors,
   anySectionsFollowed,
+  showWeather,
+  placeholder,
 }) {
   const prefs = useSelector(state => state.Prefs.values);
 
@@ -151,6 +159,8 @@ function CardSection({
   const { sectionPersonalization } = useSelector(
     state => state.DiscoveryStream
   );
+  const { isForStartupCache } = useSelector(state => state.App);
+
   const showTopics = prefs[PREF_TOPICS_ENABLED];
   const mayHaveSectionsCards = prefs[PREF_SECTIONS_CARDS_ENABLED];
   const mayHaveSectionsCardsThumbsUpDown =
@@ -159,6 +169,7 @@ function CardSection({
   const selectedTopics = prefs[PREF_TOPICS_SELECTED];
   const availableTopics = prefs[PREF_TOPICS_AVAILABLE];
   const refinedCardsLayout = prefs[PREF_REFINED_CARDS_ENABLED];
+  const spocsStartupCacheEnabled = prefs[PREF_SPOCS_STARTUPCACHE_ENABLED];
 
   const trendingEnabled =
     prefs[PREF_TRENDING_SEARCH] &&
@@ -172,7 +183,7 @@ function CardSection({
     prefs[PREF_SECTIONS_PERSONALIZATION_ENABLED];
 
   const { sectionKey, title, subtitle } = section;
-  const { responsiveLayouts } = section.layout;
+  const { responsiveLayouts, name: layoutName } = section.layout;
 
   const following = sectionPersonalization[sectionKey]?.isFollowed;
 
@@ -184,10 +195,11 @@ function CardSection({
           section: sectionKey,
           section_position: sectionPosition,
           is_section_followed: following,
+          layout_name: layoutName,
         },
       })
     );
-  }, [dispatch, sectionKey, sectionPosition, following]);
+  }, [dispatch, sectionKey, sectionPosition, following, layoutName]);
 
   // Ref to hold the section element
   const sectionRefs = useIntersectionObserver(handleIntersection);
@@ -247,7 +259,13 @@ function CardSection({
     );
   }, [dispatch, sectionPersonalization, sectionKey, sectionPosition]);
 
-  const { maxTile } = getMaxTiles(responsiveLayouts);
+  let { maxTile } = getMaxTiles(responsiveLayouts);
+  if (placeholder) {
+    // We need a number that divides evenly by 2, 3, and 4.
+    // So it can be displayed without orphans in grids with 2, 3, and 4 columns.
+    maxTile = 12;
+  }
+
   const displaySections = section.data.slice(0, maxTile);
   const isSectionEmpty = !displaySections?.length;
   const shouldShowLabels = sectionKey === "top_stories_section" && showTopics;
@@ -262,7 +280,7 @@ function CardSection({
         className={following ? "section-follow following" : "section-follow"}
       >
         {!anySectionsFollowed &&
-          sectionPosition === 1 &&
+          sectionPosition === 0 &&
           shouldShowOMCHighlight(
             messageData,
             "FollowSectionButtonHighlight"
@@ -273,11 +291,12 @@ function CardSection({
                 position="arrow-inline-start"
                 dispatch={dispatch}
                 feature="FEATURE_FOLLOW_SECTION_BUTTON"
+                messageData={messageData}
               />
             </MessageWrapper>
           )}
         {!anySectionsFollowed &&
-          sectionPosition === 1 &&
+          sectionPosition === 0 &&
           shouldShowOMCHighlight(
             messageData,
             "FollowSectionButtonAltHighlight"
@@ -323,7 +342,6 @@ function CardSection({
       />
     </div>
   );
-
   return (
     <section
       className="ds-section"
@@ -332,9 +350,12 @@ function CardSection({
       }}
     >
       <div className="section-heading">
-        <div className="section-title-wrapper">
-          <h2 className="section-title">{title}</h2>
-          {subtitle && <p className="section-subtitle">{subtitle}</p>}
+        <div className="section-heading-inline-start">
+          <div className="section-title-wrapper">
+            <h2 className="section-title">{title}</h2>
+            {subtitle && <p className="section-subtitle">{subtitle}</p>}
+          </div>
+          {showWeather && <Weather isInSection={true} />}
         </div>
         {mayHaveSectionsPersonalization ? sectionContextWrapper : null}
       </div>
@@ -348,7 +369,18 @@ function CardSection({
           );
 
           const { classNames, imageSizes } = layoutData;
-          if (!rec || rec.placeholder) {
+          // Render a placeholder card when:
+          // 1. No recommendation is available.
+          // 2. The item is flagged as a placeholder.
+          // 3. Spocs are loading for with spocs startup cache disabled.
+          if (
+            !rec ||
+            rec.placeholder ||
+            placeholder ||
+            (rec.flight_id &&
+              !spocsStartupCacheEnabled &&
+              isForStartupCache.DiscoveryStream)
+          ) {
             return <PlaceholderDSCard key={`dscard-${index}`} />;
           }
 
@@ -395,12 +427,12 @@ function CardSection({
               availableTopics={availableTopics}
               ctaButtonSponsors={ctaButtonSponsors}
               ctaButtonVariant={ctaButtonVariant}
-              spocMessageVariant={spocMessageVariant}
               sectionsClassNames={classNames.join(" ")}
               sectionsCardImageSizes={imageSizes}
               section={sectionKey}
               sectionPosition={sectionPosition}
               sectionFollowed={following}
+              sectionLayoutName={layoutName}
               isTimeSensitive={rec.isTimeSensitive}
             />
           );
@@ -421,15 +453,20 @@ function CardSections({
   dispatch,
   type,
   firstVisibleTimestamp,
-  spocMessageVariant,
   ctaButtonVariant,
   ctaButtonSponsors,
+  placeholder,
 }) {
   const prefs = useSelector(state => state.Prefs.values);
   const { spocs, sectionPersonalization } = useSelector(
     state => state.DiscoveryStream
   );
   const { messageData } = useSelector(state => state.Messages);
+  const weatherPlacement = useSelector(selectWeatherPlacement);
+  const dailyBriefSectionId =
+    prefs.trainhopConfig?.dailyBriefing?.sectionId ||
+    prefs[PREF_DAILY_BRIEF_SECTIONID];
+  const weatherEnabled = prefs.showWeather;
   const personalizationEnabled = prefs[PREF_SECTIONS_PERSONALIZATION_ENABLED];
   const interestPickerEnabled = prefs[PREF_INTEREST_PICKER_ENABLED];
 
@@ -446,7 +483,25 @@ function CardSections({
     sectionPersonalization &&
     Object.values(sectionPersonalization).some(section => section?.isFollowed);
 
-  let filteredSections = data.sections.filter(
+  let sectionsData = data.sections;
+
+  if (placeholder) {
+    // To clean up the placeholder state for sections if the whole section is loading still.
+    sectionsData = [
+      {
+        ...sectionsData[0],
+        title: "",
+        subtitle: "",
+      },
+      {
+        ...sectionsData[1],
+        title: "",
+        subtitle: "",
+      },
+    ];
+  }
+
+  let filteredSections = sectionsData.filter(
     section => !sectionPersonalization[section.sectionKey]?.isBlocked
   );
 
@@ -470,10 +525,16 @@ function CardSections({
       dispatch={dispatch}
       type={type}
       firstVisibleTimestamp={firstVisibleTimestamp}
-      spocMessageVariant={spocMessageVariant}
       ctaButtonVariant={ctaButtonVariant}
       ctaButtonSponsors={ctaButtonSponsors}
       anySectionsFollowed={anySectionsFollowed}
+      placeholder={placeholder}
+      showWeather={
+        weatherEnabled &&
+        weatherPlacement === "section" &&
+        sectionPosition === 0 &&
+        section.sectionKey === dailyBriefSectionId
+      }
     />
   ));
 
