@@ -269,7 +269,7 @@ export class TopSiteLink extends React.PureComponent {
       onClick,
       title,
       isAddButton,
-      shortcutsRefresh,
+      visibleTopSites,
     } = this.props;
 
     const topSiteOuterClassName = `top-site-outer${
@@ -292,6 +292,10 @@ export class TopSiteLink extends React.PureComponent {
     const addButtonTitlel10n = {
       "data-l10n-id": "newtab-topsites-add-shortcut-title",
     };
+    const addPinnedTitlel10n = {
+      "data-l10n-id": "topsite-label-pinned",
+      "data-l10n-args": JSON.stringify({ title }),
+    };
 
     let draggableProps = {};
     if (isDraggable) {
@@ -332,6 +336,7 @@ export class TopSiteLink extends React.PureComponent {
             reporting_url: link.sponsored_impression_url,
             advertiser: title.toLocaleLowerCase(),
             source: NEWTAB_SOURCE,
+            visible_topsites: visibleTopSites,
           }}
           // For testing.
           IntersectionObserver={this.props.IntersectionObserver}
@@ -349,6 +354,7 @@ export class TopSiteLink extends React.PureComponent {
             source: NEWTAB_SOURCE,
             isPinned: this.props.link.isPinned,
             guid: this.props.link.guid,
+            visible_topsites: visibleTopSites,
           }}
           // For testing.
           IntersectionObserver={this.props.IntersectionObserver}
@@ -380,12 +386,13 @@ export class TopSiteLink extends React.PureComponent {
             draggable={true}
             data-is-sponsored-link={!!link.sponsored_tile_id}
             onFocus={this.props.onFocus}
+            aria-label={link.isPinned ? undefined : title}
             {...(isAddButton && { ...addButtonTitlel10n })}
             {...(!isAddButton && { title })}
+            {...(link.isPinned && { ...addPinnedTitlel10n })}
+            data-l10n-args={JSON.stringify({ title })}
           >
-            {shortcutsRefresh && link.isPinned && (
-              <div className="icon icon-pin-small" />
-            )}
+            {link.isPinned && <div className="icon icon-pin-small" />}
             <div className="tile" aria-hidden={true}>
               <div
                 className={
@@ -405,9 +412,6 @@ export class TopSiteLink extends React.PureComponent {
                   />
                 )}
               </div>
-              {!shortcutsRefresh && link.searchTopSite && (
-                <div className="top-site-icon search-topsite" />
-              )}
             </div>
             <div
               className={`title${link.isPinned ? " has-icon pinned" : ""}${
@@ -421,10 +425,7 @@ export class TopSiteLink extends React.PureComponent {
                 dir="auto"
                 {...(isAddButton && { ...addButtonLabell10n })}
               >
-                {!shortcutsRefresh && link.isPinned && (
-                  <div className="icon icon-pin-small" />
-                )}
-                {shortcutsRefresh && link.searchTopSite && (
+                {link.searchTopSite && (
                   <div className="top-site-icon search-topsite" />
                 )}
                 {title || <br />}
@@ -444,6 +445,7 @@ export class TopSiteLink extends React.PureComponent {
                 dispatch={this.props.dispatch}
                 feature="FEATURE_SHORTCUT_HIGHLIGHT"
                 position="inset-block-end inset-inline-start"
+                messageData={this.props.Messages?.messageData}
               />
             </MessageWrapper>
           )}
@@ -516,6 +518,7 @@ export class TopSite extends React.PureComponent {
           type: at.OPEN_LINK,
           data: Object.assign(this.props.link, {
             event: { altKey, button, ctrlKey, metaKey, shiftKey },
+            is_sponsored: !!this.props.link.sponsored_tile_id,
           }),
         })
       );
@@ -579,6 +582,7 @@ export class TopSite extends React.PureComponent {
               reporting_url: this.props.link.sponsored_click_url,
               advertiser: title.toLocaleLowerCase(),
               source: NEWTAB_SOURCE,
+              visible_topsites: this.props.visibleTopSites,
             },
           })
         );
@@ -593,6 +597,7 @@ export class TopSite extends React.PureComponent {
               source: NEWTAB_SOURCE,
               isPinned: this.props.link.isPinned,
               guid: this.props.link.guid,
+              visible_topsites: this.props.visibleTopSites,
             },
           })
         );
@@ -948,8 +953,6 @@ export class _TopSiteList extends React.PureComponent {
 
   render() {
     const { props } = this;
-    const prefs = props.Prefs.values;
-    const shortcutsRefresh = prefs["newtabShortcuts.refresh"];
     const topSites = this.state.topSitesPreview || this._getTopSites();
     const topSitesUI = [];
     const commonProps = {
@@ -1008,6 +1011,7 @@ export class _TopSiteList extends React.PureComponent {
               this.onTopsiteFocus(i);
             }}
             Messages={this.props.Messages}
+            visibleTopSites={this.props.visibleTopSites}
           />
         );
       } else {
@@ -1019,7 +1023,6 @@ export class _TopSiteList extends React.PureComponent {
             {...slotProps}
             {...commonProps}
             colors={props.colors}
-            shortcutsRefresh={shortcutsRefresh}
             setRef={
               i === this.state.focusedIndex
                 ? el => {
@@ -1031,6 +1034,7 @@ export class _TopSiteList extends React.PureComponent {
             onFocus={() => {
               this.onTopsiteFocus(i);
             }}
+            visibleTopSites={this.props.visibleTopSites}
           />
         );
       }
