@@ -12,6 +12,7 @@ import {
   button,
   footer,
 } from "devtools/client/shared/vendor/react-dom-factories";
+import SearchInput from "../shared/SearchInput";
 import EventListeners from "../shared/EventListeners";
 import { connect } from "devtools/client/shared/vendor/react-redux";
 import {
@@ -26,11 +27,25 @@ import {
   getIsCurrentlyTracing,
   getRuntimeVersions,
   getTraceHighlightedDomEvents,
+  getTraceMatchingSearchTraces,
+  getTraceMatchingSearchException,
+  getTraceMatchingSearchValueOrGrip,
+  getIsTracingValues,
 } from "../../selectors/index";
+import { NO_SEARCH_VALUE } from "../../reducers/tracer-frames";
+
+const { throttle } = require("resource://devtools/shared/throttle.js");
 const VirtualizedTree = require("resource://devtools/client/shared/components/VirtualizedTree.js");
 const FrameView = createFactory(
   require("resource://devtools/client/shared/components/Frame.js")
 );
+const Reps = ChromeUtils.importESModule(
+  "resource://devtools/client/shared/components/reps/index.mjs"
+);
+const {
+  REPS: { Rep },
+  MODE,
+} = Reps;
 const {
   TRACER_FIELDS_INDEXES,
 } = require("resource://devtools/server/actors/tracer.js");
@@ -38,10 +53,10 @@ const {
   HTMLTooltip,
 } = require("resource://devtools/client/shared/widgets/tooltip/HTMLTooltip.js");
 
-const {
-  TabPanel,
-  Tabs,
-} = require("resource://devtools/client/shared/components/tabs/Tabs.js");
+const { TabPanel, Tabs } = ChromeUtils.importESModule(
+  "resource://devtools/client/shared/components/tabs/Tabs.mjs",
+  { global: "current" }
+);
 
 import actions from "../../actions/index";
 
@@ -76,6 +91,12 @@ export class Tracer extends Component {
     this.onSliderClick = this.onSliderClick.bind(this);
     this.onSliderWheel = this.onSliderWheel.bind(this);
     this.resetZoom = this.resetZoom.bind(this);
+
+    // Throttle requests to update the trace argument search query as it is a costly operation
+    this.throttledUpdateSearch = throttle(
+      this.throttledUpdateSearch.bind(this),
+      250
+    );
   }
 
   UNSAFE_componentWillReceiveProps(nextProps) {
@@ -125,6 +146,18 @@ export class Tracer extends Component {
     if (!this.tooltip) {
       this.instantiateTooltip();
     }
+
+    // Force updating indexes when we navigate back to the tracer sidebar.
+    // For example, when we navigated away to the source tree.
+    if (!this.state.renderedTraceCount) {
+      this.updateIndexes(
+        {
+          startIndex: this.state.startIndex,
+          endIndex: this.state.endIndex,
+        },
+        this.props
+      );
+    }
   }
 
   instantiateTooltip() {
@@ -182,7 +215,7 @@ export class Tracer extends Component {
     }
   }
 
-  renderTree() {
+  renderCallTree() {
     let {
       selectedTraceIndex,
       topTraces,
@@ -395,7 +428,9 @@ export class Tracer extends Component {
             className,
             showFunctionName: true,
             showAnonymousFunctionName: true,
-            frame,
+            // Frame's savedFrameToLocation mess up with the frame object
+            // by incrementing the column unexpectedly.
+            frame: { ...frame, column: frame.column + 1 },
             sourceMapURLService: window.sourceMapURLService,
           })
         );
@@ -578,6 +613,11 @@ export class Tracer extends Component {
   }
 
   renderEventsInSlider() {
+    // When getting back to tracer sidebar after having moved to any other side panel, like source tree,
+    // the timeline is null and would crash here.
+    if (!this.refs.timeline) {
+      return null;
+    }
     const { topTraces, allTraces, traceChildren } = this.props;
     const { startIndex, endIndex } = this.state;
 
@@ -731,6 +771,112 @@ export class Tracer extends Component {
     );
   }
 
+  searchInputOnChange = e => {
+    const searchString = e.target.value;
+
+    // Throttle the calls to searchTraceArgument as that a costly operation
+    this.throttledUpdateSearch(searchString);
+  };
+
+  throttledUpdateSearch(searchString) {
+    this.props.searchTraceArguments(searchString);
+  }
+
+  /**
+   * Select the next or previous trace according to the current search string
+   *
+   * @param {Boolean} goForward
+   *                  Select the next matching trace if true,
+   *                  otherwise select the previous one.
+   */
+  selectNextMatchingTrace = goForward => {
+    const { tracesMatchingSearch, allTraces } = this.props;
+    const selectedTrace = allTraces[this.props.selectedTraceIndex];
+    const currentIndexInMatchingArray =
+      tracesMatchingSearch.indexOf(selectedTrace);
+
+    let nextIndexInMatchingArray;
+    if (goForward) {
+      // If we aren't selecting any of the matching traces, or the last one,
+      // select the first matching trace.
+      if (
+        currentIndexInMatchingArray == -1 ||
+        currentIndexInMatchingArray == tracesMatchingSearch.length - 1
+      ) {
+        nextIndexInMatchingArray = 0;
+      } else {
+        nextIndexInMatchingArray = currentIndexInMatchingArray + 1;
+      }
+    } else if (
+      currentIndexInMatchingArray == -1 ||
+      currentIndexInMatchingArray == 0
+    ) {
+      nextIndexInMatchingArray = tracesMatchingSearch.length - 1;
+    } else {
+      nextIndexInMatchingArray = currentIndexInMatchingArray - 1;
+    }
+
+    // `selectTrace` expect a trace index (and not a trace object)
+    const nextTraceIndex = allTraces.indexOf(
+      tracesMatchingSearch[nextIndexInMatchingArray]
+    );
+
+    this.props.selectTrace(nextTraceIndex);
+  };
+
+  renderCallTreeSearchInput() {
+    const { tracesMatchingSearch, searchExceptionMessage, searchValueOrGrip } =
+      this.props;
+    return [
+      React.createElement(SearchInput, {
+        count: tracesMatchingSearch.length,
+
+        placeholder: this.props.traceValues
+          ? `Search for function call argument values ("foo", 42, $0, $("canvas"), …)`
+          : "Enable tracing values to search for values",
+        disabled: !this.props.traceValues,
+        size: "small",
+        showClose: false,
+        onChange: this.searchInputOnChange,
+        onKeyDown: e => {
+          if (e.key == "Enter") {
+            // Shift key will reverse the selection direction
+            this.selectNextMatchingTrace(!e.shiftKey);
+          }
+        },
+        handlePrev: () => this.selectNextMatchingTrace(false),
+        handleNext: () => this.selectNextMatchingTrace(true),
+      }),
+
+      // When this isn't a valid primitive type, we try to evaluate on the server
+      // and show the exception, if one was thrown
+      searchExceptionMessage
+        ? div({ className: "search-exception" }, searchExceptionMessage)
+        : null,
+
+      // When we have a valid search string, either matching a primitive type or an object,
+      // we display it here, alongside the number of matches
+      this.props.traceValues && searchValueOrGrip != NO_SEARCH_VALUE
+        ? div(
+            { className: "search-value" },
+            "Searching for:",
+            Rep({
+              object: searchValueOrGrip,
+              mode: MODE.SHORT,
+              onDOMNodeClick: () =>
+                this.props.openElementInInspector(searchValueOrGrip),
+              onInspectIconClick: () =>
+                this.props.openElementInInspector(searchValueOrGrip),
+              onDOMNodeMouseOver: () =>
+                this.props.highlightDomElement(searchValueOrGrip),
+              onDOMNodeMouseOut: () => this.props.unHighlightDomElement(),
+            }),
+            ` (${tracesMatchingSearch.length} match(es))`
+          )
+        : null,
+    ];
+  }
+
   render() {
     const { runtimeVersions } = this.props;
 
@@ -777,7 +923,11 @@ export class Tracer extends Component {
             id: "tracer-traces",
             title: "Call Traces",
           },
-          this.renderTree()
+          div(
+            { className: "call-tree-container" },
+            ...this.renderCallTreeSearchInput(),
+            this.renderCallTree()
+          )
         ),
         React.createElement(
           TabPanel,
@@ -897,9 +1047,17 @@ const mapStateToProps = state => {
     selectedTraceIndex: getSelectedTraceIndex(state),
     runtimeVersions: getRuntimeVersions(state),
     highlightedDomEvents: getTraceHighlightedDomEvents(state),
+    tracesMatchingSearch: getTraceMatchingSearchTraces(state),
+    searchExceptionMessage: getTraceMatchingSearchException(state),
+    searchValueOrGrip: getTraceMatchingSearchValueOrGrip(state),
+    traceValues: getIsTracingValues(state),
   };
 };
 
 export default connect(mapStateToProps, {
   selectTrace: actions.selectTrace,
+  searchTraceArguments: actions.searchTraceArguments,
+  openElementInInspector: actions.openElementInInspectorCommand,
+  highlightDomElement: actions.highlightDomElement,
+  unHighlightDomElement: actions.unHighlightDomElement,
 })(Tracer);
