@@ -16,6 +16,7 @@ import {
   teardownWorkers,
 } from "./utils/bootstrap";
 
+import { initialTabState } from "./reducers/tabs";
 import { initialBreakpointsState } from "./reducers/breakpoints";
 import { initialSourcesState } from "./reducers/sources";
 import { initialSourcesTreeState } from "./reducers/sources-tree";
@@ -26,6 +27,11 @@ import { initialEventListenerState } from "./reducers/event-listeners";
 const {
   sanitizeBreakpoints,
 } = require("resource://devtools/client/shared/thread-utils.js");
+const {
+  START_IGNORE_ACTION,
+} = require("resource://devtools/client/shared/redux/middleware/ignore.js");
+
+let gStore;
 
 async function syncBreakpoints() {
   const breakpoints = await asyncStore.pendingBreakpoints;
@@ -72,7 +78,10 @@ async function loadInitialState(commands) {
   const pendingBreakpoints = sanitizeBreakpoints(
     await asyncStore.pendingBreakpoints
   );
-  const tabs = { tabs: await asyncStore.tabs };
+  const tabs = initialTabState({
+    urls: await asyncStore.openedURLs,
+    prettyPrintedURLs: new Set(await asyncStore.prettyPrintedURLs),
+  });
   const xhrBreakpoints = await asyncStore.xhrBreakpoints;
   const blackboxedRanges = await asyncStore.blackboxedRanges;
   const eventListenerBreakpoints = await asyncStore.eventListenerBreakpoints;
@@ -85,6 +94,7 @@ async function loadInitialState(commands) {
   const sourceBlackBox = initialSourceBlackBoxState({ blackboxedRanges });
   const sources = initialSourcesState();
   const sourcesTree = initialSourcesTreeState({
+    mainThreadProjectDirectoryRoots: await asyncStore.directoryRoots,
     isWebExtension: commands.descriptorFront.isWebExtensionDescriptor,
   });
   const ui = initialUIState();
@@ -123,6 +133,7 @@ export async function bootstrap({
     panel,
     initialState
   );
+  gStore = store;
 
   const connected = firefox.onConnect(
     commands,
@@ -154,7 +165,18 @@ export async function bootstrap({
 }
 
 export async function destroy() {
+  // Instruct redux to start ignoring any further action
+  if (gStore) {
+    gStore.dispatch(START_IGNORE_ACTION);
+    gStore = null;
+  }
+
+  // Unregister all listeners set on DevTools RDP client
   firefox.onDisconnect();
+
+  // Unmount all React components
   unmountRoot();
+
+  // Unregister and cleanup everything about parser, pretty print and source map workers
   teardownWorkers();
 }
