@@ -81,8 +81,9 @@ const PREF_DRAGGABLE = "devtools.inspector.draggable_properties";
 const PREF_INPLACE_EDITOR_FOCUS_NEXT_ON_ENTER =
   "devtools.inspector.rule-view.focusNextOnEnter";
 const FILTER_CHANGED_TIMEOUT = 150;
-// Removes the flash-out class from an element after 1 second.
-const PROPERTY_FLASHING_DURATION = 1000;
+// Removes the flash-out class from an element after 1 second (100ms in tests so they
+// don't take too long to run).
+const PROPERTY_FLASHING_DURATION = flags.testing ? 100 : 1000;
 
 // This is used to parse user input when filtering.
 const FILTER_PROP_RE = /\s*([^:\s]*)\s*:\s*(.*?)\s*;?$/;
@@ -174,7 +175,6 @@ function CssRuleView(inspector, document, store) {
   this._onToggleDarkColorSchemeSimulation =
     this._onToggleDarkColorSchemeSimulation.bind(this);
   this._onTogglePrintSimulation = this._onTogglePrintSimulation.bind(this);
-  this.highlightElementRule = this.highlightElementRule.bind(this);
   this.highlightProperty = this.highlightProperty.bind(this);
   this.refreshPanel = this.refreshPanel.bind(this);
 
@@ -401,7 +401,7 @@ CssRuleView.prototype = {
       this.inspector.sidebar &&
       this.inspector.toolbox.currentToolId === "inspector" &&
       (this.inspector.sidebar.getCurrentTabID() == "ruleview" ||
-        this.inspector.is3PaneModeEnabled)
+        this.inspector.isThreePaneModeEnabled)
     );
   },
 
@@ -837,79 +837,97 @@ CssRuleView.prototype = {
 
   /**
    * Called when the user enters a search term in the filter style search box.
+   * The actual filtering (done in _doFilterStyles) will be throttled if the search input
+   * isn't empty, but will happen immediately when the search gets cleared.
    */
   _onFilterStyles() {
     if (this._filterChangedTimeout) {
       clearTimeout(this._filterChangedTimeout);
     }
 
-    const filterTimeout = this.searchValue.length ? FILTER_CHANGED_TIMEOUT : 0;
-    this.searchClearButton.hidden = this.searchValue.length === 0;
+    const isSearchEmpty = this.searchValue.length === 0;
+    this.searchClearButton.hidden = isSearchEmpty;
 
-    this._filterChangedTimeout = setTimeout(() => {
-      this.searchData = {
-        searchPropertyMatch: FILTER_PROP_RE.exec(this.searchValue),
-        searchPropertyName: this.searchValue,
-        searchPropertyValue: this.searchValue,
-        strictSearchValue: "",
-        strictSearchPropertyName: false,
-        strictSearchPropertyValue: false,
-        strictSearchAllValues: false,
-      };
+    // If the search is cleared update the UI directly so calls to this function (or any
+    // callsite of it) can assume the UI is up to date directly after the call.
+    if (isSearchEmpty) {
+      this._doFilterStyles();
+    } else {
+      this._filterChangedTimeout = setTimeout(
+        () => this._doFilterStyles(),
+        FILTER_CHANGED_TIMEOUT
+      );
+    }
+  },
 
-      if (this.searchData.searchPropertyMatch) {
-        // Parse search value as a single property line and extract the
-        // property name and value. If the parsed property name or value is
-        // contained in backquotes (`), extract the value within the backquotes
-        // and set the corresponding strict search for the property to true.
-        if (FILTER_STRICT_RE.test(this.searchData.searchPropertyMatch[1])) {
-          this.searchData.strictSearchPropertyName = true;
-          this.searchData.searchPropertyName = FILTER_STRICT_RE.exec(
-            this.searchData.searchPropertyMatch[1]
-          )[1];
-        } else {
-          this.searchData.searchPropertyName =
-            this.searchData.searchPropertyMatch[1];
-        }
+  /**
+   * Actually update search data and update the UI to reflect the current search.
+   *
+   * @emits ruleview-filtered
+   */
+  _doFilterStyles() {
+    this.searchData = {
+      searchPropertyMatch: FILTER_PROP_RE.exec(this.searchValue),
+      searchPropertyName: this.searchValue,
+      searchPropertyValue: this.searchValue,
+      strictSearchValue: "",
+      strictSearchPropertyName: false,
+      strictSearchPropertyValue: false,
+      strictSearchAllValues: false,
+    };
 
-        if (FILTER_STRICT_RE.test(this.searchData.searchPropertyMatch[2])) {
-          this.searchData.strictSearchPropertyValue = true;
-          this.searchData.searchPropertyValue = FILTER_STRICT_RE.exec(
-            this.searchData.searchPropertyMatch[2]
-          )[1];
-        } else {
-          this.searchData.searchPropertyValue =
-            this.searchData.searchPropertyMatch[2];
-        }
-
-        // Strict search for stylesheets will match the property line regex.
-        // Extract the search value within the backquotes to be used
-        // in the strict search for stylesheets in _highlightStyleSheet.
-        if (FILTER_STRICT_RE.test(this.searchValue)) {
-          this.searchData.strictSearchValue = FILTER_STRICT_RE.exec(
-            this.searchValue
-          )[1];
-        }
-      } else if (FILTER_STRICT_RE.test(this.searchValue)) {
-        // If the search value does not correspond to a property line and
-        // is contained in backquotes, extract the search value within the
-        // backquotes and set the flag to perform a strict search for all
-        // the values (selector, stylesheet, property and computed values).
-        const searchValue = FILTER_STRICT_RE.exec(this.searchValue)[1];
-        this.searchData.strictSearchAllValues = true;
-        this.searchData.searchPropertyName = searchValue;
-        this.searchData.searchPropertyValue = searchValue;
-        this.searchData.strictSearchValue = searchValue;
+    if (this.searchData.searchPropertyMatch) {
+      // Parse search value as a single property line and extract the
+      // property name and value. If the parsed property name or value is
+      // contained in backquotes (`), extract the value within the backquotes
+      // and set the corresponding strict search for the property to true.
+      if (FILTER_STRICT_RE.test(this.searchData.searchPropertyMatch[1])) {
+        this.searchData.strictSearchPropertyName = true;
+        this.searchData.searchPropertyName = FILTER_STRICT_RE.exec(
+          this.searchData.searchPropertyMatch[1]
+        )[1];
+      } else {
+        this.searchData.searchPropertyName =
+          this.searchData.searchPropertyMatch[1];
       }
 
-      this._clearHighlight(this.element);
-      this._clearRules();
-      this._createEditors();
+      if (FILTER_STRICT_RE.test(this.searchData.searchPropertyMatch[2])) {
+        this.searchData.strictSearchPropertyValue = true;
+        this.searchData.searchPropertyValue = FILTER_STRICT_RE.exec(
+          this.searchData.searchPropertyMatch[2]
+        )[1];
+      } else {
+        this.searchData.searchPropertyValue =
+          this.searchData.searchPropertyMatch[2];
+      }
 
-      this.inspector.emit("ruleview-filtered");
+      // Strict search for stylesheets will match the property line regex.
+      // Extract the search value within the backquotes to be used
+      // in the strict search for stylesheets in _highlightStyleSheet.
+      if (FILTER_STRICT_RE.test(this.searchValue)) {
+        this.searchData.strictSearchValue = FILTER_STRICT_RE.exec(
+          this.searchValue
+        )[1];
+      }
+    } else if (FILTER_STRICT_RE.test(this.searchValue)) {
+      // If the search value does not correspond to a property line and
+      // is contained in backquotes, extract the search value within the
+      // backquotes and set the flag to perform a strict search for all
+      // the values (selector, stylesheet, property and computed values).
+      const searchValue = FILTER_STRICT_RE.exec(this.searchValue)[1];
+      this.searchData.strictSearchAllValues = true;
+      this.searchData.searchPropertyName = searchValue;
+      this.searchData.searchPropertyValue = searchValue;
+      this.searchData.strictSearchValue = searchValue;
+    }
 
-      this._filterChangeTimeout = null;
-    }, filterTimeout);
+    this._clearHighlight(this.element);
+    this._clearRules();
+    this._createEditors();
+
+    this.inspector.emit("ruleview-filtered");
+
+    this._filterChangeTimeout = null;
   },
 
   /**
@@ -1086,6 +1104,9 @@ CssRuleView.prototype = {
       return Promise.resolve(undefined);
     }
 
+    const isProfilerActive = Services.profiler?.IsActive();
+    const startTime = isProfilerActive ? ChromeUtils.now() : null;
+
     this.pageStyle = element.inspectorFront.pageStyle;
     this.pageStyle.on("stylesheet-updated", this.refreshPanel);
 
@@ -1132,6 +1153,17 @@ CssRuleView.prototype = {
             this._changed();
           };
         }
+        if (isProfilerActive && this._elementStyle.rules) {
+          let declarations = 0;
+          for (const rule of this._elementStyle.rules) {
+            declarations += rule.textProps.length;
+          }
+          ChromeUtils.addProfilerMarker(
+            "DevTools:CssRuleView.selectElement",
+            startTime,
+            `${declarations} CSS declarations in ${this._elementStyle.rules.length} rules`
+          );
+        }
       })
       .catch(e => {
         if (this._elementStyle === elementStyle) {
@@ -2051,25 +2083,28 @@ CssRuleView.prototype = {
    *
    * @param  {Element} element
    *         The element.
+   * @returns {Promise} Promise that resolves after the element was flashed-out
    */
   _flashElement(element) {
     flashElementOn(element, {
       backgroundClass: "theme-bg-contrast",
     });
 
-    if (this._flashMutationTimer) {
-      clearTimeout(this._removeFlashOutTimer);
-      this._flashMutationTimer = null;
+    if (this._flashMutationCallback) {
+      this._flashMutationCallback();
     }
 
-    this._flashMutationTimer = setTimeout(() => {
-      flashElementOff(element, {
-        backgroundClass: "theme-bg-contrast",
-      });
+    return new Promise(resolve => {
+      this._flashMutationCallback = () => {
+        flashElementOff(element, {
+          backgroundClass: "theme-bg-contrast",
+        });
+        this._flashMutationCallback = null;
+        resolve();
+      };
 
-      // Emit "scrolled-to-property" for use by tests.
-      this.emit("scrolled-to-element");
-    }, PROPERTY_FLASHING_DURATION);
+      setTimeout(this._flashMutationCallback, PROPERTY_FLASHING_DURATION);
+    });
   },
 
   /**
@@ -2119,66 +2154,38 @@ CssRuleView.prototype = {
     this._toggleContainerVisibility(toggle, container, true, true);
   },
 
-  /**
-   * Finds the rule with the matching actorID and highlights it.
-   *
-   * @param  {String} ruleId
-   *         The actorID of the rule.
-   */
-  highlightElementRule(ruleId) {
-    let scrollBehavior = "smooth";
-
-    const rule = this.rules.find(r => r.domRule.actorID === ruleId);
-
-    if (!rule) {
-      return;
-    }
-
-    if (rule.domRule.actorID === ruleId) {
-      // If using 2-Pane mode, then switch to the Rules tab first.
-      if (!this.inspector.is3PaneModeEnabled) {
-        this.inspector.sidebar.select("ruleview");
-      }
-
-      if (rule.pseudoElement.length && !this.showPseudoElements) {
-        scrollBehavior = "auto";
-        this._togglePseudoElementRuleContainer();
-      }
-
-      const {
-        editor: { element },
-      } = rule;
-
-      // Scroll to the top of the rule and highlight it.
-      this._scrollToElement(element, null, scrollBehavior);
-      this._flashElement(element);
-    }
-  },
-
   /**
    * Finds the specified TextProperty name in the rule view. If found, scroll to and
    * flash the TextProperty.
    *
    * @param  {String} name
    *         The property name to scroll to and highlight.
+   * @param  {Object} options
+   * @param  {Function|undefined} options.ruleValidator
+   *         An optional function that can be used to filter out rules we shouldn't look
+   *         into to find the property name. The function is called with a Rule object,
+   *         and the rule will be skipped if the function returns a falsy value.
    * @return {Boolean} true if the TextProperty name is found, and false otherwise.
    */
-  highlightProperty(name) {
+  highlightProperty(name, { ruleValidator } = {}) {
+    // First, let's clear any search we might have, as the property could be hidden
+    this._onClearSearch();
+
+    let scrollBehavior = "auto";
+    const hasRuleValidator = typeof ruleValidator === "function";
     for (const rule of this.rules) {
+      if (hasRuleValidator && !ruleValidator(rule)) {
+        continue;
+      }
       for (const textProp of rule.textProps) {
         if (textProp.overridden || textProp.invisible || !textProp.enabled) {
           continue;
         }
 
-        const {
-          editor: { selectorText },
-        } = rule;
-        let scrollBehavior = "smooth";
-
         // First, search for a matching authored property.
         if (textProp.name === name) {
           // If using 2-Pane mode, then switch to the Rules tab first.
-          if (!this.inspector.is3PaneModeEnabled) {
+          if (!this.inspector.isThreePaneModeEnabled) {
             this.inspector.sidebar.select("ruleview");
           }
 
@@ -2191,15 +2198,11 @@ CssRuleView.prototype = {
             this._togglePseudoElementRuleContainer();
           }
 
-          // Scroll to the top of the property's rule so that both the property and its
-          // rule are visible.
-          this._scrollToElement(
-            selectorText,
+          this._highlightElementInRule(
+            rule,
             textProp.editor.element,
             scrollBehavior
           );
-          this._flashElement(textProp.editor.element);
-
           return true;
         }
 
@@ -2210,7 +2213,7 @@ CssRuleView.prototype = {
           }
 
           if (computed.name === name) {
-            if (!this.inspector.is3PaneModeEnabled) {
+            if (!this.inspector.isThreePaneModeEnabled) {
               this.inspector.sidebar.select("ruleview");
             }
 
@@ -2225,22 +2228,63 @@ CssRuleView.prototype = {
             // Expand the computed list.
             textProp.editor.expandForFilter();
 
-            this._scrollToElement(
-              selectorText,
+            this._highlightElementInRule(
+              rule,
               computed.element,
               scrollBehavior
             );
-            this._flashElement(computed.element);
 
             return true;
           }
         }
       }
     }
+    // If the property is a CSS variable and we didn't find its declaration, it might
+    // be a registered property
+    if (name.startsWith("--")) {
+      // Get a potential @property section
+      const propertyContainer = this.styleDocument.getElementById(
+        REGISTERED_PROPERTIES_CONTAINER_ID
+      );
+      if (propertyContainer) {
+        const propertyEl = propertyContainer.querySelector(
+          `[data-name="${name}"]`
+        );
+        if (propertyEl) {
+          const toggle = this.styleDocument.querySelector(
+            `[aria-controls="${REGISTERED_PROPERTIES_CONTAINER_ID}"]`
+          );
+          if (toggle.ariaExpanded === "false") {
+            this._toggleContainerVisibility(toggle, propertyContainer);
+          }
+
+          this._highlightElementInRule(null, propertyEl, scrollBehavior);
+        }
+        return true;
+      }
+    }
 
     return false;
   },
 
+  /**
+   * Highlight a given element in a rule editor
+   *
+   * @param {Rule} rule
+   * @param {Element} element
+   * @param {String} scrollBehavior
+   */
+  _highlightElementInRule(rule, element, scrollBehavior) {
+    if (rule) {
+      this._scrollToElement(rule.editor.selectorText, element, scrollBehavior);
+    } else {
+      this._scrollToElement(element, null, scrollBehavior);
+    }
+    this._flashElement(element).then(() =>
+      this.emitForTests("element-highlighted", element)
+    );
+  },
+
   /**
    * Returns a Map (keyed by name) of the registered
    * properties for the currently selected node document.
