@@ -9,7 +9,7 @@ const {
   PSEUDO_CLASSES,
 } = require("resource://devtools/shared/css/constants.js");
 const {
-  style: { ELEMENT_STYLE },
+  style: { ELEMENT_STYLE, PRES_HINTS },
 } = require("resource://devtools/shared/constants.js");
 const Rule = require("resource://devtools/client/inspector/rules/models/rule.js");
 const {
@@ -78,7 +78,7 @@ function RuleEditor(ruleView, rule, options = {}) {
   this.rule = rule;
   this.options = options;
 
-  this.isEditable = !rule.isSystem;
+  this.isEditable = rule.isEditable();
   // Flag that blocks updates of the selector and properties when it is
   // being edited
   this.isEditing = false;
@@ -101,6 +101,10 @@ function RuleEditor(ruleView, rule, options = {}) {
 
 RuleEditor.prototype = {
   destroy() {
+    for (const prop of this.rule.textProps) {
+      prop.editor?.destroy();
+    }
+
     this.rule.domRule.off("location-changed");
     this.toolbox.off("tool-registered", this._onToolChanged);
     this.toolbox.off("tool-unregistered", this._onToolChanged);
@@ -144,7 +148,10 @@ RuleEditor.prototype = {
 
     // Add the source link for supported rules. inline style and pres hints are not visible
     // in the StyleEditor, so don't show anything for such rule.
-    if (this.rule.domRule.type !== ELEMENT_STYLE) {
+    if (
+      this.rule.domRule.type !== ELEMENT_STYLE &&
+      this.rule.domRule.type !== PRES_HINTS
+    ) {
       this.source = createChild(this.element, "div", {
         class: "ruleview-rule-source theme-link",
       });
@@ -331,6 +338,13 @@ RuleEditor.prototype = {
       tabindex: this.isSelectorEditable ? "0" : "-1",
     });
 
+    if (
+      this.rule.domRule.type === ELEMENT_STYLE ||
+      this.rule.domRule.type === PRES_HINTS
+    ) {
+      this.selectorText.classList.add("alternative-selector");
+    }
+
     if (this.isSelectorEditable) {
       this.selectorText.addEventListener("click", event => {
         // Clicks within the selector shouldn't propagate any further.
@@ -618,14 +632,15 @@ RuleEditor.prototype = {
    */
   populate(reset) {
     // Clear out existing viewers.
-    while (this.selectorText.hasChildNodes()) {
-      this.selectorText.removeChild(this.selectorText.lastChild);
-    }
+    this.selectorText.replaceChildren();
 
     // If selector text comes from a css rule, highlight selectors that
     // actually match.  For custom selector text (such as for the 'element'
     // style, just show the text directly.
-    if (this.rule.domRule.type === ELEMENT_STYLE) {
+    if (
+      this.rule.domRule.type === ELEMENT_STYLE ||
+      this.rule.domRule.type === PRES_HINTS
+    ) {
       this.selectorText.textContent = this.rule.selectorText;
     } else if (this.rule.domRule.type === CSSRule.KEYFRAME_RULE) {
       this.selectorText.textContent = this.rule.domRule.keyText;
@@ -644,9 +659,7 @@ RuleEditor.prototype = {
         focusedElSelector = CssLogic.findCssSelector(this.doc.activeElement);
       }
 
-      while (this.propertyList.hasChildNodes()) {
-        this.propertyList.removeChild(this.propertyList.lastChild);
-      }
+      this.propertyList.replaceChildren();
     }
 
     for (const prop of this.rule.textProps) {
@@ -662,7 +675,9 @@ RuleEditor.prototype = {
       }
     }
 
-    if (focusedElSelector) {
+    // Set focus if the focus is still in the current document (avoid stealing
+    // the focus, see Bug 1911627).
+    if (this.doc.hasFocus() && focusedElSelector) {
       const elementToFocus = this.doc.querySelector(focusedElSelector);
       if (elementToFocus && this.element.contains(elementToFocus)) {
         // We need to wait for a tick for the focus to be properly set
