@@ -1,6 +1,6 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
-/* eslint no-unused-vars: [2, {"vars": "local"}] */
+/* eslint no-unused-vars: [2, {"vars": "local", caughtErrors: "none"}] */
 
 /* import-globals-from ../../inspector/test/shared-head.js */
 
@@ -10,6 +10,49 @@
 // and we start using it in xpcshell tests as well.
 // It contains various common helper functions.
 
+// Recording already set preferences.
+const devtoolsPreferences = Services.prefs.getBranch("devtools");
+const alreadySetPreferences = new Set();
+for (const pref of devtoolsPreferences.getChildList("")) {
+  if (devtoolsPreferences.prefHasUserValue(pref)) {
+    alreadySetPreferences.add(pref);
+  }
+}
+
+{
+  const { PromiseTestUtils } = ChromeUtils.importESModule(
+    "resource://testing-common/PromiseTestUtils.sys.mjs"
+  );
+  PromiseTestUtils.allowMatchingRejectionsGlobally(
+    /REDUX_MIDDLEWARE_IGNORED_REDUX_ACTION/
+  );
+}
+
+async function resetPreferencesModifiedDuringTest() {
+  if (!isXpcshell) {
+    await SpecialPowers.flushPrefEnv();
+  }
+
+  // Reset devtools preferences modified by the test.
+  for (const pref of devtoolsPreferences.getChildList("")) {
+    if (
+      devtoolsPreferences.prefHasUserValue(pref) &&
+      !alreadySetPreferences.has(pref)
+    ) {
+      devtoolsPreferences.clearUserPref(pref);
+    }
+  }
+
+  // Cleanup some generic Firefox preferences set indirectly by tests.
+  for (const pref of [
+    "browser.firefox-view.view-count",
+    "extensions.ui.lastCategory",
+    "sidebar.old-sidebar.has-used",
+  ]) {
+    Services.prefs.clearUserPref(pref);
+  }
+}
+
 const isMochitest = "gTestPath" in this;
 const isXpcshell = !isMochitest;
 if (isXpcshell) {
@@ -25,9 +68,8 @@ const { Constructor: CC } = Components;
 // and allocation sites if DEBUG_DEVTOOLS_ALLOCATIONS is set to "verbose".
 const DEBUG_ALLOCATIONS = Services.env.get("DEBUG_DEVTOOLS_ALLOCATIONS");
 if (DEBUG_ALLOCATIONS) {
-  // Use a custom loader with `invisibleToDebugger` flag for the allocation tracker
-  // as it instantiates custom Debugger API instances and has to be running in a distinct
-  // compartments from DevTools and system scopes (JSMs, XPCOM,...)
+  // Load the allocation tracker from the distinct privileged loader in order
+  // to be able to debug all privileged code (ESMs, XPCOM,...) running in the shared privileged global.
   const {
     useDistinctSystemPrincipalLoader,
     releaseDistinctSystemPrincipalLoader,
@@ -72,9 +114,8 @@ if (DEBUG_ALLOCATIONS) {
 // in any other way on parent process's main thread.
 const DEBUG_STEP = Services.env.get("DEBUG_STEP");
 if (DEBUG_STEP) {
-  // Use a custom loader with `invisibleToDebugger` flag for the allocation tracker
-  // as it instantiates custom Debugger API instances and has to be running in a distinct
-  // compartments from DevTools and system scopes (JSMs, XPCOM,...)
+  // Load the stepper code from the distinct privileged loader in order
+  // to be able to debug all privileged code (ESMs, XPCOM,...) running in the shared privileged global.
   const {
     useDistinctSystemPrincipalLoader,
     releaseDistinctSystemPrincipalLoader,
@@ -97,9 +138,8 @@ if (DEBUG_STEP) {
 
 const DEBUG_TRACE_LINE = Services.env.get("DEBUG_TRACE_LINE");
 if (DEBUG_TRACE_LINE) {
-  // Use a custom loader with `invisibleToDebugger` flag for the allocation tracker
-  // as it instantiates custom Debugger API instances and has to be running in a distinct
-  // compartments from DevTools and system scopes (ESMs, XPCOM,...)
+  // Load the tracing code from the distinct privileged loader in order
+  // to be able to debug all privileged code (ESMs, XPCOM,...) running in the shared privileged global.
   const {
     useDistinctSystemPrincipalLoader,
     releaseDistinctSystemPrincipalLoader,
@@ -273,11 +313,13 @@ registerCleanupFunction(() => {
   ConsoleAPIStorage.removeLogEventListener(onConsoleMessage);
 });
 
-Services.prefs.setBoolPref("devtools.inspector.three-pane-enabled", true);
-
 // Disable this preference to reduce exceptions related to pending `listWorkers`
 // requests occuring after a process is created/destroyed. See Bug 1620983.
-Services.prefs.setBoolPref("dom.ipc.processPrelaunch.enabled", false);
+add_setup(async () => {
+  if (!isXpcshell) {
+    await pushPref("dom.ipc.processPrelaunch.enabled", false);
+  }
+});
 
 // On some Linux platforms, prefers-reduced-motion is enabled, which would
 // trigger the notification to be displayed in the toolbox. Dismiss the message
@@ -287,18 +329,8 @@ Services.prefs.setBoolPref(
   true
 );
 
-registerCleanupFunction(() => {
-  Services.prefs.clearUserPref("devtools.dump.emit");
-  Services.prefs.clearUserPref("devtools.inspector.three-pane-enabled");
-  Services.prefs.clearUserPref("dom.ipc.processPrelaunch.enabled");
-  Services.prefs.clearUserPref("devtools.toolbox.host");
-  Services.prefs.clearUserPref("devtools.toolbox.previousHost");
-  Services.prefs.clearUserPref("devtools.toolbox.splitconsole.open");
-  Services.prefs.clearUserPref("devtools.toolbox.splitconsoleHeight");
-  Services.prefs.clearUserPref(
-    "devtools.inspector.simple-highlighters.message-dismissed"
-  );
-});
+// Enable dumping scope variables when a test failure occurs.
+Services.prefs.setBoolPref("devtools.testing.testScopes", true);
 
 var {
   BrowserConsoleManager,
@@ -337,6 +369,10 @@ registerCleanupFunction(async function cleanup() {
       conn.close();
     }
   }
+
+  // Reset all preferences AFTER the toolbox is closed.
+  // NOTE: Doing it before toolbox destruction could trigger observers.
+  await resetPreferencesModifiedDuringTest();
 });
 
 async function safeCloseBrowserConsole({ clearOutput = false } = {}) {
@@ -537,7 +573,10 @@ async function addTab(url, options = {}) {
   }
 
   if (waitForLoad) {
-    await BrowserTestUtils.browserLoaded(tab.linkedBrowser);
+    // accept any URL as url arg might not be serialized or redirects might happen
+    await BrowserTestUtils.browserLoaded(tab.linkedBrowser, {
+      wantLoad: () => true,
+    });
     // Waiting for presShell helps with test timeouts in webrender platforms.
     await waitForPresShell(tab.linkedBrowser);
     info("Tab added and finished loading");
@@ -984,17 +1023,6 @@ async function createAndAttachTargetForTab(tab) {
   return target;
 }
 
-function isFissionEnabled() {
-  return SpecialPowers.useRemoteSubframes;
-}
-
-function isEveryFrameTargetEnabled() {
-  return Services.prefs.getBoolPref(
-    "devtools.every-frame-target.enabled",
-    false
-  );
-}
-
 /**
  * Open the inspector in a tab with given URL.
  * @param {string} url  The URL to open.
@@ -1442,7 +1470,12 @@ function pushPref(preferenceName, value) {
   return SpecialPowers.pushPrefEnv(options);
 }
 
-async function closeToolbox() {
+/**
+ * Close the toolbox for the selected tab if needed.
+ */
+async function closeToolboxIfOpen() {
+  // `closeToolboxForTab` will be a noop if the selected tab does not have any
+  // toolbox.
   await gDevTools.closeToolboxForTab(gBrowser.selectedTab);
 }
 
@@ -1613,14 +1646,15 @@ function colorAt(image, x, y) {
 
 let allDownloads = [];
 /**
- * Returns a Promise that resolves when a new screenshot is available in the download folder.
+ * Returns a Promise that resolves when a new file (e.g. screenshot, JSON, …) is available
+ * in the download folder.
  *
  * @param {Object} [options]
- * @param {Boolean} options.isWindowPrivate: Set to true if the window from which the screenshot
- *                  is taken is a private window. This will ensure that we check that the
- *                  screenshot appears in the private window, not the non-private one (See Bug 1783373)
+ * @param {Boolean} options.isWindowPrivate: Set to true if the window from which the file
+ *                  is downloaded is a private window. This will ensure that we check that the
+ *                  file appears in the private window, not the non-private one (See Bug 1783373)
  */
-async function waitUntilScreenshot({ isWindowPrivate = false } = {}) {
+async function waitUntilDownload({ isWindowPrivate = false } = {}) {
   const { Downloads } = ChromeUtils.importESModule(
     "resource://gre/modules/Downloads.sys.mjs"
   );
@@ -1683,7 +1717,7 @@ async function takeNodeScreenshot(inspector) {
   info(
     "Call screenshotNode() and wait until the screenshot is found in the Downloads"
   );
-  const whenScreenshotSucceeded = waitUntilScreenshot();
+  const whenScreenshotSucceeded = waitUntilDownload();
   inspector.screenshotNode();
   const filePath = await whenScreenshotSucceeded;
 
@@ -2532,3 +2566,137 @@ async function _maybeOpenAncestorMenu(menuItem) {
   parentMenu.openMenu(true);
   await shown;
 }
+
+/**
+ * Returns the list of console messages DOM Element display in the Web Console
+ * which contains a given string.
+ *
+ * @param {Toolbox} toolbox
+ * @param {String} query
+ * @return {Array<DOMElement>}
+ */
+async function findConsoleMessages(toolbox, query) {
+  const webConsole = await toolbox.getPanel("webconsole");
+  const win = webConsole._frameWindow;
+  return Array.prototype.filter.call(
+    win.document.querySelectorAll(".message"),
+    e => e.innerText.includes(query)
+  );
+}
+
+/**
+ * Wait for a console message to appear with a given text and a given link to
+ * a specific location in a JS source.
+ * Returns the DOM Element in the Web Console for the link to the JS Source.
+ *
+ * @param {Toolbox} toolbox
+ * @param {String} messageText
+ * @param {String} linkText
+ * @return {DOMElement}
+ */
+async function waitForConsoleMessageLink(toolbox, messageText, linkText) {
+  await toolbox.selectTool("webconsole");
+
+  return waitFor(async () => {
+    // Wait until the message updates.
+    const [message] = await findConsoleMessages(toolbox, messageText);
+    if (!message) {
+      return false;
+    }
+    const linkEl = message.querySelector(".frame-link-source");
+    if (!linkEl || linkEl.textContent !== linkText) {
+      return false;
+    }
+
+    return linkEl;
+  });
+}
+
+/**
+ * Click on a Frame component link and ensure it opens the debugger on the expected location
+ *
+ * @param {Toolbox} toolbox
+ * @param {DOMElement} frameLinkNode
+ * @param {Object] options
+ * @param {String|null} options.url
+ * @param {Number|null} options.line
+ * @param {Number|null} options.column
+ * @param {String|undefined} logPointExpr
+ */
+async function clickAndAssertFrameLinkNode(
+  toolbox,
+  frameLinkNode,
+  { url, line, column },
+  logPointExpr
+) {
+  info("checking click on node location");
+
+  // If the debugger hasn't fully loaded yet and breakpoints are still being
+  // added when we click on the logpoint link, the logpoint panel might not
+  // render. Work around this for now, see bug 1592854.
+  if (logPointExpr) {
+    await waitForTime(1000);
+  }
+
+  const onSourceOpened = toolbox.once("source-opened-in-debugger");
+
+  EventUtils.sendMouseEvent(
+    { type: "click" },
+    // The frame DOM Element may be coming from a Debugger Frame component, or a shared compoentn Frame component
+    // and the link would be at a distinct selector.
+    frameLinkNode.querySelector(".frame-link-filename") ||
+      frameLinkNode.querySelector(".location")
+  );
+
+  // Wait for the source to finish loading, if it is pending.
+  await onSourceOpened;
+
+  // Wait for the debugger to have fully processed the opened source
+  const dbg = toolbox.getPanel("jsdebugger");
+
+  const selectedLocation = await waitFor(() =>
+    dbg._selectors.getSelectedLocation(dbg._getState())
+  );
+
+  if (typeof url == "string") {
+    const frameUrl = frameLinkNode.getAttribute("data-url");
+    is(frameUrl, url, "Frame link url is correct");
+
+    is(selectedLocation.source.url, url, "debugger opened url is correct");
+  }
+  if (typeof line == "number") {
+    const frameLine = frameLinkNode.getAttribute("data-line");
+    is(parseInt(frameLine, 10), line, "Frame link line is correct");
+
+    is(selectedLocation.line, line, "debugger opened line is correct");
+  }
+  if (typeof column == "number") {
+    // Note that debugger's Frame component doesn't show the column
+    const frameColumn = frameLinkNode.getAttribute("data-column");
+    is(parseInt(frameColumn, 10), column, "Frame link column is correct");
+
+    // Redux location object uses 0-based column, while we display a 1-based one.
+    is(
+      selectedLocation.column + 1,
+      column,
+      "debugger opened column is correct"
+    );
+  }
+
+  if (logPointExpr !== undefined && logPointExpr !== "") {
+    const inputEl = dbg.panelWin.document.activeElement;
+
+    const isPanelFocused =
+      inputEl.classList.contains("cm-content") &&
+      inputEl.closest(".conditional-breakpoint-panel.log-point");
+
+    ok(isPanelFocused, "The textarea of logpoint panel is focused");
+
+    const inputValue = inputEl.parentElement.parentElement.innerText.trim();
+    is(
+      inputValue,
+      logPointExpr,
+      "The input in the open logpoint panel matches the logpoint expression"
+    );
+  }
+}
