@@ -1,6 +1,5 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
-/* eslint no-unused-vars: [2, {"vars": "local"}] */
 
 "use strict";
 
@@ -46,15 +45,6 @@ registerCleanupFunction(async function () {
   // Reset all cookies, tests loading sjs_slow-response-test-server.sjs will
   // set a foo cookie which might have side effects on other tests.
   Services.cookies.removeAll();
-
-  Services.prefs.clearUserPref("devtools.webconsole.ui.filterbar");
-
-  // Reset all filter prefs between tests. First flushPrefEnv in case one of the
-  // filter prefs has been pushed for the test
-  await SpecialPowers.flushPrefEnv();
-  Services.prefs.getChildList("devtools.webconsole.filter").forEach(pref => {
-    Services.prefs.clearUserPref(pref);
-  });
 });
 
 /**
@@ -279,6 +269,18 @@ async function waitForMessageByType(hud, text, typeSelector) {
   return messages[0];
 }
 
+/**
+ * Wait for the Source editor to be available.
+ *
+ * @param {Object} panel
+ * @returns
+ */
+async function waitForSourceEditor(panel) {
+  return waitUntil(() => {
+    return !!panel.querySelector(".cm-editor");
+  });
+}
+
 /**
  * Execute an input expression.
  *
@@ -530,25 +532,14 @@ function waitForNodeMutation(node, observeConfig = {}) {
  *                         @see findMessageByType.
  *        - typeSelector: {string} A part of selector for the message, to
  *                                 specify the message type.
- *        - expectUrl: {boolean} Whether the URL in the opened source should
- *                               match the link, or whether it is expected to
- *                               be null.
- *        - expectLine: {boolean} It indicates if there is the need to check
- *                                the line.
- *        - expectColumn: {boolean} It indicates if there is the need to check
- *                                the column.
+ *        - url : {String|null} URL expected to be opened.
+ *        - line : {Number|null} Line expected to be opened.
+ *        - column : {Number|null} Column expected to be opened.
  *        - logPointExpr: {String} The logpoint expression
  */
 async function testOpenInDebugger(
   hud,
-  {
-    text,
-    typeSelector,
-    expectUrl = true,
-    expectLine = true,
-    expectColumn = true,
-    logPointExpr = undefined,
-  }
+  { text, typeSelector, url, column, line, logPointExpr = undefined }
 ) {
   info(`Finding message for open-in-debugger test; text is "${text}"`);
   const messageNode = await waitFor(() =>
@@ -556,104 +547,15 @@ async function testOpenInDebugger(
   );
   const locationNode = messageNode.querySelector(".message-location");
   ok(locationNode, "The message does have a location link");
-  await checkClickOnNode(
-    hud,
+
+  await clickAndAssertFrameLinkNode(
     hud.toolbox,
     locationNode,
-    expectUrl,
-    expectLine,
-    expectColumn,
+    { url, column, line },
     logPointExpr
   );
 }
 
-/**
- * Helper function for testOpenInDebugger.
- */
-async function checkClickOnNode(
-  hud,
-  toolbox,
-  frameLinkNode,
-  expectUrl,
-  expectLine,
-  expectColumn,
-  logPointExpr
-) {
-  info("checking click on node location");
-
-  // If the debugger hasn't fully loaded yet and breakpoints are still being
-  // added when we click on the logpoint link, the logpoint panel might not
-  // render. Work around this for now, see bug 1592854.
-  await waitForTime(1000);
-
-  const onSourceInDebuggerOpened = once(hud, "source-in-debugger-opened");
-
-  EventUtils.sendMouseEvent(
-    { type: "click" },
-    frameLinkNode.querySelector(".frame-link-filename")
-  );
-
-  await onSourceInDebuggerOpened;
-
-  const dbg = toolbox.getPanel("jsdebugger");
-
-  // Wait for the source to finish loading, if it is pending.
-  await waitFor(
-    () =>
-      !!dbg._selectors.getSelectedSource(dbg._getState()) &&
-      !!dbg._selectors.getSelectedLocation(dbg._getState())
-  );
-
-  if (expectUrl) {
-    const url = frameLinkNode.getAttribute("data-url");
-    ok(url, `source url found ("${url}")`);
-
-    is(
-      dbg._selectors.getSelectedSource(dbg._getState()).url,
-      url,
-      "expected source url"
-    );
-  }
-  if (expectLine) {
-    const line = frameLinkNode.getAttribute("data-line");
-    ok(line, `source line found ("${line}")`);
-
-    is(
-      parseInt(dbg._selectors.getSelectedLocation(dbg._getState()).line, 10),
-      parseInt(line, 10),
-      "expected source line"
-    );
-  }
-  if (expectColumn) {
-    const column = frameLinkNode.getAttribute("data-column");
-    ok(column, `source column found ("${column}")`);
-
-    is(
-      parseInt(dbg._selectors.getSelectedLocation(dbg._getState()).column, 10),
-      parseInt(column, 10),
-      "expected source column"
-    );
-  }
-
-  if (logPointExpr !== undefined && logPointExpr !== "") {
-    const inputEl = dbg.panelWin.document.activeElement;
-
-    const isPanelFocused = isCm6Enabled
-      ? inputEl.classList.contains("cm-content") &&
-        inputEl.closest(".conditional-breakpoint-panel.log-point")
-      : inputEl.tagName == "TEXTAREA";
-
-    ok(isPanelFocused, "The textarea of logpoint panel is focused");
-
-    const inputValue = inputEl.parentElement.parentElement.innerText.trim();
-    is(
-      inputValue,
-      logPointExpr,
-      "The input in the open logpoint panel matches the logpoint expression"
-    );
-  }
-}
-
 /**
  * Returns true if the give node is currently focused.
  */
@@ -1490,7 +1392,7 @@ async function selectFrame(dbg, frame) {
   await onScopes;
 }
 
-async function pauseDebugger(dbg, options = { shouldWaitForLoadScopes: true }) {
+async function pauseDebugger(dbg, options) {
   info("Waiting for debugger to pause");
   const onPaused = waitForPaused(dbg, null, options);
   SpecialPowers.spawn(gBrowser.selectedBrowser, [], function () {
@@ -1826,6 +1728,7 @@ function checkContextSelectorMenu(hud, expected) {
  * @param {String} expected.tooltip: The tooltip of the target element in the menu
  * @param {Boolean} expected.checked: if the target should be selected or not
  * @param {Boolean} expected.separator: if the element is a simple separator
+ * @param {Boolean} expected.indented: if the element is indented
  */
 function checkContextSelectorMenuItemAt(hud, index, expected) {
   const el = getContextSelectorItems(hud).at(index);
@@ -1838,6 +1741,7 @@ function checkContextSelectorMenuItemAt(hud, index, expected) {
   const elChecked = el.getAttribute("aria-checked") === "true";
   const elTooltip = el.getAttribute("title");
   const elLabel = el.querySelector(".label").innerText;
+  const indented = el.classList.contains("indented");
 
   is(elLabel, expected.label, `The item has the expected label`);
   is(elTooltip, expected.tooltip, `Item "${elLabel}" has the expected tooltip`);
@@ -1846,6 +1750,11 @@ function checkContextSelectorMenuItemAt(hud, index, expected) {
     expected.checked,
     `Item "${elLabel}" is ${expected.checked ? "checked" : "unchecked"}`
   );
+  is(
+    indented,
+    expected.indented ?? false,
+    `Item "${elLabel}" is ${!indented ? " not" : ""} indented`
+  );
 }
 
 /**
