@@ -135,18 +135,17 @@ class RootActor extends Actor {
             "dom.worker.console.dispatch_events_to_main_thread"
           )
         : true,
-      // @backward-compat { version 142 } Process Descriptor's `getWatcher()`
+      // @backward-compat { version 144 } Process Descriptor's `getWatcher()`
       // supports a new 'enableWindowGlobalThreadActors' flag to enable
       // the WindowGlobal's thread actors when debugging the whole browser.
       // This was actually changed in 137, but we support it for VSCode until
       // ESR 140 is the only ESR available.
-      // This should happen when Firefox 143 gets released.
-      // Contact Holger Benl (hbenl) to make sure the VS Code extension is
-      // updated.
+      //
+      // ESR 128 EOL is currently planned for September 16 2025. Do not remove
+      // this trait before that data AND make sure the extension has been
+      // updated (https://github.com/firefox-devtools/vscode-firefox-debug/issues/391).
+      // Contact Holger Benl (hbenl) for topics related to the extension.
       supportsEnableWindowGlobalThreadActors: true,
-      // @backward-compat { version 140 } Use the bulk API to transfer the
-      // performance profile data.
-      useBulkTransferForPerformanceProfile: true,
     };
   }
 
