@@ -21,11 +21,6 @@ const TEXT_WRAP_BALANCE_LIMIT = Services.prefs.getIntPref(
   10
 );
 
-const ALIGN_CONTENT_BLOCKS = Services.prefs.getBoolPref(
-  "layout.css.align-content.blocks.enabled",
-  false
-);
-
 const VISITED_MDN_LINK = "https://developer.mozilla.org/docs/Web/CSS/:visited";
 const VISITED_INVALID_PROPERTIES = allCssPropertiesExcept([
   "all",
@@ -138,8 +133,8 @@ class InactivePropertyHelper {
    * If you add a new rule, also add a test for it in:
    * server/tests/chrome/test_inspector-inactive-property-helper.html
    *
-   * The main export is `isPropertyUsed()`, which can be used to check if a
-   * property is used or not, and why.
+   * The main export is `getInactiveCssDataForProperty()`, which can be used to check if a
+   * property is inactive or not, and why.
    *
    * NOTE: We should generally *not* add rules here for any CSS properties that
    * inherit by default, because it's hard for us to know whether such
@@ -241,20 +236,15 @@ class InactivePropertyHelper {
             "inline-flex",
             "grid",
             "inline-grid",
+            "block",
+            "inline-block",
             // Uncomment table-cell when Bug 1883357 is fixed.
             // "table-cell"
           ];
-          if (ALIGN_CONTENT_BLOCKS) {
-            supportedDisplay.push("block", "inline-block");
-          }
           return !this.checkComputedStyle("display", supportedDisplay);
         },
-        fixId: ALIGN_CONTENT_BLOCKS
-          ? "inactive-css-not-grid-or-flex-or-block-container-fix"
-          : "inactive-css-not-grid-or-flex-container-fix",
-        msgId: ALIGN_CONTENT_BLOCKS
-          ? "inactive-css-property-because-of-display"
-          : "inactive-css-not-grid-or-flex-container",
+        fixId: "inactive-css-not-grid-or-flex-or-block-container-fix",
+        msgId: "inactive-css-property-because-of-display",
       },
       // column-gap and shorthands used on non-grid or non-flex or non-multi-col container.
       {
@@ -397,6 +387,29 @@ class InactivePropertyHelper {
         fixId: "inactive-css-not-block-fix",
         msgId: "inactive-css-not-block",
       },
+      // Block container properties used on non-block-container elements.
+      {
+        invalidProperties: ["text-overflow"],
+        when: () => !this.isBlockContainer(),
+        fixId: "inactive-css-not-block-container-fix",
+        msgId: "inactive-css-not-block-container",
+      },
+      // Block, flex, and grid container properties used on non-block, non-flex or non-grid container elements.
+      {
+        invalidProperties: [
+          "overflow",
+          "overflow-block",
+          "overflow-inline",
+          "overflow-x",
+          "overflow-y",
+        ],
+        when: () =>
+          !this.isBlockContainer() &&
+          !this.flexContainer &&
+          !this.gridContainer,
+        fixId: "inactive-css-not-block-flex-grid-container-fix",
+        msgId: "inactive-css-not-block-flex-grid-container",
+      },
       // shape-image-threshold, shape-margin, shape-outside properties used on non-floated elements.
       {
         invalidProperties: [
@@ -453,6 +466,19 @@ class InactivePropertyHelper {
         fixId: "inactive-text-overflow-when-no-overflow-fix",
         msgId: "inactive-text-overflow-when-no-overflow",
       },
+      // content-visibility used on elements for which size containment doesn't apply.
+      {
+        invalidProperties: ["content-visibility"],
+        when: () =>
+          !this.hasPrincipalBox ||
+          this.table ||
+          this.internalTableElement ||
+          this.rubyContainer ||
+          this.internalRubyElement ||
+          this.nonAtomicInlineBox,
+        fixId: "inactive-css-no-size-containment-fix",
+        msgId: "inactive-css-no-size-containment",
+      },
       // margin properties used on table internal elements.
       {
         invalidProperties: [
@@ -618,6 +644,13 @@ class InactivePropertyHelper {
         fixId: "learn-more",
         msgId: "inactive-css-no-width-height",
       },
+      // anchor-name used on element not creating a principal box.
+      {
+        invalidProperties: ["anchor-name"],
+        when: () => !this.hasPrincipalBox,
+        fixId: "inactive-css-no-principal-box-fix",
+        msgId: "inactive-css-no-principal-box",
+      },
     ];
   }
 
@@ -647,7 +680,7 @@ class InactivePropertyHelper {
    * If you add a new rule, also add a test for it in:
    * server/tests/chrome/test_inspector-inactive-property-helper.html
    *
-   * The main export is `isPropertyUsed()`, which can be used to check if a
+   * The main export is `getInactiveCssDataForProperty()`, which can be used to check if a
    * property is used or not, and why.
    */
   ACCEPTED_PROPERTIES_VALIDATORS = [
@@ -782,7 +815,8 @@ class InactivePropertyHelper {
    * @param {String} property
    *        The CSS property name.
    *
-   * @return {Object} object
+   * @return {Object|null} object
+   *         if the property is active, this will return null
    * @return {String} object.display
    *         The element computed display value.
    * @return {String} object.fixId
@@ -796,13 +830,11 @@ class InactivePropertyHelper {
    * @return {String} object.learnMoreURL
    *         An optional link if we need to open an other link than
    *         the default MDN property one.
-   * @return {Boolean} object.used
-   *         true if the property is used.
    */
-  isPropertyUsed(el, elStyle, cssRule, property) {
+  getInactiveCssDataForProperty(el, elStyle, cssRule, property) {
     // Assume the property is used when the Inactive CSS pref is not enabled
     if (!INACTIVE_CSS_ENABLED) {
-      return { used: true };
+      return null;
     }
 
     let fixId = "";
@@ -853,7 +885,7 @@ class InactivePropertyHelper {
     // in the accepted properties validators, assume the property is used.
     if (!isNotAccepted && !this.invalidProperties.has(property)) {
       this.unselect();
-      return { used: true };
+      return null;
     }
 
     // Otherwise, if there was no issue from the accepted properties validators,
@@ -871,6 +903,10 @@ class InactivePropertyHelper {
       display = elStyle ? elStyle.display : null;
     } catch (e) {}
 
+    if (used) {
+      return null;
+    }
+
     return {
       display,
       fixId,
@@ -878,7 +914,6 @@ class InactivePropertyHelper {
       property,
       learnMoreURL,
       lineCount,
-      used,
     };
   }
 
@@ -973,6 +1008,13 @@ class InactivePropertyHelper {
     ]);
   }
 
+  /**
+   *  Check if the current node is an block container.
+   */
+  isBlockContainer() {
+    return this.node ? InspectorUtils.isBlockContainer(this.node) : false;
+  }
+
   /**
    *  Check if the current node is an inline-level box.
    */
@@ -1040,6 +1082,13 @@ class InactivePropertyHelper {
     return !!this.getParentMultiColElement(this.node);
   }
 
+  /**
+   * Check if the current node is a table.
+   */
+  get table() {
+    return this.checkComputedStyle("display", ["table", "inline-table"]);
+  }
+
   /**
    * Check if the current node is a table row.
    */
@@ -1155,6 +1204,25 @@ class InactivePropertyHelper {
     return isVerticalRowGroup || isVerticalColumnGroup;
   }
 
+  /**
+   * Check if the current node is a ruby container.
+   */
+  get rubyContainer() {
+    return this.checkComputedStyle("display", ["ruby"]);
+  }
+
+  /**
+   * Check if the current node is an internal ruby element.
+   */
+  get internalRubyElement() {
+    return this.checkComputedStyle("display", [
+      "ruby-base",
+      "ruby-text",
+      "ruby-base-container",
+      "ruby-text-container",
+    ]);
+  }
+
   /**
    * Returns whether this element uses CSS layout.
    */
@@ -1174,6 +1242,30 @@ class InactivePropertyHelper {
     );
   }
 
+  /**
+   * Check if the current node is a non-atomic CSS inline box.
+   */
+  get nonAtomicInlineBox() {
+    return (
+      this.hasCssLayout &&
+      this.nonReplaced &&
+      this.style &&
+      this.checkComputedStyle("display", ["inline", "inline list-item"])
+    );
+  }
+
+  /**
+   * Check if the current node generates a principal box.
+   */
+  get hasPrincipalBox() {
+    return (
+      this.hasCssLayout &&
+      this.style &&
+      this.style.display !== "none" &&
+      this.style.display !== "contents"
+    );
+  }
+
   /**
    * Check if the current selector refers to a ::first-letter pseudo-element
    */
@@ -1604,7 +1696,8 @@ function computedStyle(node, window = node.ownerGlobal) {
 
 const inactivePropertyHelper = new InactivePropertyHelper();
 
-// The only public method from this module is `isPropertyUsed`.
-exports.isPropertyUsed = inactivePropertyHelper.isPropertyUsed.bind(
-  inactivePropertyHelper
-);
+// The only public method from this module is `getInactiveCssDataForProperty`.
+exports.getInactiveCssDataForProperty =
+  inactivePropertyHelper.getInactiveCssDataForProperty.bind(
+    inactivePropertyHelper
+  );
