@@ -236,8 +236,6 @@ exports.WatcherActor = class WatcherActor extends Actor {
       traits: {
         ...this.sessionContext.supportedTargets,
         resources: this.sessionContext.supportedResources,
-        // @backward-compat { version 142 } Supports emitting of multiple network event updates.
-        multipleNetworkEventUpdates: true,
       },
     };
   }
@@ -579,7 +577,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
     );
 
     switch (this.sessionContext.type) {
-      case "all":
+      case "all": {
         const parentProcessTargetActor = actors.find(
           actor => actor.typeName === "parentProcessTarget"
         );
@@ -587,6 +585,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
           return new Set([parentProcessTargetActor]);
         }
         return new Set();
+      }
       case "browser-element":
       case "webextension":
         // All target actors for browser-element and webextension sessions
